"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCompileData = void 0;
const assert_1 = require("tsafe/assert");
const exclude_1 = require("tsafe/exclude");
const memoizee_1 = __importDefault(require("memoizee"));
function createCompileData(params) {
    const { getSoftwareExternalData: getSoftwareExternalData_params, comptoirDuLibreApi, getCnllPrestatairesSill, getSoftwareLatestVersion: getSoftwareLatestVersion_params, getServiceProviders } = params;
    const compileData = async (params) => {
        let { db: { softwareRows, agentRows, softwareReferentRows, softwareUserRows, instanceRows }, getCachedSoftware } = params;
        if (getCachedSoftware === undefined) {
            getServiceProviders.clear();
            getCnllPrestatairesSill.clear();
            comptoirDuLibreApi.getComptoirDuLibre.clear();
        }
        const [{ softwares: cdlSoftwares }, cnllPrestatairesSill, serviceProvidersBySillId] = await Promise.all([
            comptoirDuLibreApi.getComptoirDuLibre(),
            getCnllPrestatairesSill(),
            getServiceProviders()
        ]);
        const { partialSoftwareBySillId } = await (async () => {
            var _a;
            const partialSoftwareBySillId = {};
            const getExternalSoftware = (0, memoizee_1.default)(async (wikidataId) => {
                if (getCachedSoftware === undefined) {
                    getSoftwareExternalData_params.clear(wikidataId);
                }
                return await getSoftwareExternalData_params(wikidataId);
            }, { "promise": true });
            const getSoftwareLatestVersion = (0, memoizee_1.default)(async (repoUrl) => {
                if (getCachedSoftware === undefined) {
                    getSoftwareLatestVersion_params.clear(repoUrl);
                }
                return await getSoftwareLatestVersion_params(repoUrl, getCachedSoftware === undefined ? "look everywhere" : "quick");
            }, { "promise": true });
            for (const { id: sillId, name, externalId, comptoirDuLibreId, parentSoftwareWikidataId, similarSoftwareExternalDataIds } of softwareRows) {
                const cache = getCachedSoftware === null || getCachedSoftware === void 0 ? void 0 : getCachedSoftware({ "sillSoftwareId": sillId });
                if (cache === undefined) {
                    console.log(`Scrapping the web for info about ${name}`);
                }
                const externalSoftware_prev = cache === null || cache === void 0 ? void 0 : cache.softwareExternalData;
                const externalSoftware = externalId === undefined
                    ? undefined
                    : ((_a = cache === null || cache === void 0 ? void 0 : cache.softwareExternalData) === null || _a === void 0 ? void 0 : _a.externalId) === externalId
                        ? cache.softwareExternalData
                        : await getExternalSoftware(externalId);
                const cdlSoftware_prev = cdlSoftwares.find(({ id }) => { var _a; return ((_a = cache === null || cache === void 0 ? void 0 : cache.comptoirDuLibreSoftware) === null || _a === void 0 ? void 0 : _a.id) === id; });
                const cdlSoftware = cdlSoftwares.find(({ id }) => comptoirDuLibreId === id);
                const [latestVersion, comptoirDuLibreLogoUrl, comptoirDuLibreKeywords, parentWikidataSoftware, similarExternalSoftwares, instances] = await Promise.all([
                    (async () => {
                        const getRepoUrl = (wikidataSoftware, cdlSoftware) => { var _a, _b, _c; return (_c = (_a = wikidataSoftware === null || wikidataSoftware === void 0 ? void 0 : wikidataSoftware.sourceUrl) !== null && _a !== void 0 ? _a : (_b = cdlSoftware === null || cdlSoftware === void 0 ? void 0 : cdlSoftware.external_resources) === null || _b === void 0 ? void 0 : _b.repository) !== null && _c !== void 0 ? _c : undefined; };
                        const repoUrl_prev = getRepoUrl(externalSoftware_prev, cdlSoftware_prev);
                        const repoUrl = getRepoUrl(externalSoftware, cdlSoftware);
                        if (repoUrl_prev === repoUrl) {
                            return cache === null || cache === void 0 ? void 0 : cache.latestVersion;
                        }
                        if (repoUrl === undefined) {
                            return undefined;
                        }
                        return getSoftwareLatestVersion(repoUrl);
                    })(),
                    (() => {
                        var _a;
                        if ((cdlSoftware_prev === null || cdlSoftware_prev === void 0 ? void 0 : cdlSoftware_prev.id) === (cdlSoftware === null || cdlSoftware === void 0 ? void 0 : cdlSoftware.id)) {
                            return (_a = cache === null || cache === void 0 ? void 0 : cache.comptoirDuLibreSoftware) === null || _a === void 0 ? void 0 : _a.logoUrl;
                        }
                        if (cdlSoftware === undefined) {
                            return undefined;
                        }
                        return comptoirDuLibreApi.getIconUrl({ "comptoirDuLibreId": cdlSoftware.id });
                    })(),
                    (() => {
                        var _a;
                        if ((cdlSoftware_prev === null || cdlSoftware_prev === void 0 ? void 0 : cdlSoftware_prev.id) === (cdlSoftware === null || cdlSoftware === void 0 ? void 0 : cdlSoftware.id)) {
                            return (_a = cache === null || cache === void 0 ? void 0 : cache.comptoirDuLibreSoftware) === null || _a === void 0 ? void 0 : _a.keywords;
                        }
                        if (cdlSoftware === undefined) {
                            return undefined;
                        }
                        return comptoirDuLibreApi.getKeywords({ "comptoirDuLibreId": cdlSoftware.id });
                    })(),
                    (async () => {
                        var _a;
                        if (((_a = cache === null || cache === void 0 ? void 0 : cache.parentWikidataSoftware) === null || _a === void 0 ? void 0 : _a.externalId) === parentSoftwareWikidataId) {
                            return cache === null || cache === void 0 ? void 0 : cache.parentWikidataSoftware;
                        }
                        if (parentSoftwareWikidataId === undefined) {
                            return undefined;
                        }
                        const parentWikidataSoftware = await getExternalSoftware(parentSoftwareWikidataId);
                        if (parentWikidataSoftware === undefined) {
                            return undefined;
                        }
                        return {
                            "externalId": parentWikidataSoftware.externalId,
                            "label": parentWikidataSoftware.label,
                            "description": parentWikidataSoftware.description
                        };
                    })(),
                    Promise.all(similarSoftwareExternalDataIds.map(async (externalId) => {
                        var _a;
                        {
                            const similarSoftware_cache = (_a = cache === null || cache === void 0 ? void 0 : cache.similarExternalSoftwares) === null || _a === void 0 ? void 0 : _a.find(similarSoftware => similarSoftware.externalId === externalId);
                            if (similarSoftware_cache !== undefined) {
                                return similarSoftware_cache;
                            }
                        }
                        const wikidataSoftware = await getExternalSoftware(externalId);
                        if (wikidataSoftware === undefined) {
                            return undefined;
                        }
                        return {
                            "externalId": wikidataSoftware.externalId,
                            "label": wikidataSoftware.label,
                            "description": wikidataSoftware.description,
                            "isLibreSoftware": wikidataSoftware.isLibreSoftware,
                            "externalDataOrigin": wikidataSoftware.externalDataOrigin
                        };
                    })).then(similarWikidataSoftwares => similarWikidataSoftwares.filter((0, exclude_1.exclude)(undefined))),
                    Promise.all(instanceRows
                        .filter(({ mainSoftwareSillId }) => mainSoftwareSillId === sillId)
                        .map(async ({ id, otherSoftwareWikidataIds }) => {
                        const instance_cache = cache === null || cache === void 0 ? void 0 : cache.instances.find(instance => instance.id === id);
                        return {
                            "id": id,
                            "otherWikidataSoftwares": await Promise.all(otherSoftwareWikidataIds.map(async (wikidataId) => {
                                const otherWikidataSoftware_cache = instance_cache === null || instance_cache === void 0 ? void 0 : instance_cache.otherWikidataSoftwares.find(wikidataSoftware => wikidataSoftware.externalId === wikidataId);
                                if (otherWikidataSoftware_cache !== undefined) {
                                    return otherWikidataSoftware_cache;
                                }
                                const wikidataSoftware = await getExternalSoftware(wikidataId);
                                if (wikidataSoftware === undefined) {
                                    return undefined;
                                }
                                return {
                                    "externalId": wikidataSoftware.externalId,
                                    "label": wikidataSoftware.label,
                                    "description": wikidataSoftware.description
                                };
                            })).then(otherWikidataSoftwares => otherWikidataSoftwares.filter((0, exclude_1.exclude)(undefined)))
                        };
                    }))
                ]);
                partialSoftwareBySillId[sillId] = {
                    softwareExternalData: externalSoftware,
                    latestVersion,
                    comptoirDuLibreSoftware: cdlSoftware === undefined
                        ? undefined
                        : {
                            "id": cdlSoftware.id,
                            "logoUrl": comptoirDuLibreLogoUrl,
                            "keywords": comptoirDuLibreKeywords
                        },
                    parentWikidataSoftware,
                    similarExternalSoftwares,
                    instances
                };
            }
            return { partialSoftwareBySillId };
        })();
        const compiledData = softwareRows
            .map(softwareRow => ({
            softwareRow,
            "referents": softwareReferentRows
                .filter(({ softwareId }) => softwareId === softwareRow.id)
                .map(({ agentEmail, softwareId, ...rest }) => ({
                ...(() => {
                    const agentRow = agentRows.find(({ email }) => email === agentEmail);
                    (0, assert_1.assert)(agentRow !== undefined, `problem with referent ${agentEmail}`);
                    return {
                        "email": agentRow.email,
                        "organization": agentRow.organization
                    };
                })(),
                "isExpert": rest.isExpert,
                "serviceUrl": rest.serviceUrl,
                "useCaseDescription": rest.useCaseDescription
            })),
            "users": softwareUserRows
                .filter(({ softwareId }) => softwareId === softwareRow.id)
                .map(({ agentEmail, softwareId, ...rest }) => ({
                ...(() => {
                    const agentRow = agentRows.find(({ email }) => email === agentEmail);
                    (0, assert_1.assert)(agentRow !== undefined, `problem with user ${agentEmail}`);
                    return {
                        "organization": agentRow.organization
                    };
                })(),
                "os": rest.os,
                "serviceUrl": rest.serviceUrl,
                "useCaseDescription": rest.useCaseDescription,
                "version": rest.version
            }))
        }))
            .map(({ softwareRow: { "id": sillId, name, description, referencedSinceTime, updateTime, dereferencing, isStillInObservation, doRespectRgaa, isFromFrenchPublicService, isPresentInSupportContract, comptoirDuLibreId, license, softwareType, catalogNumeriqueGouvFrId, versionMin, workshopUrls, testUrls, categories, generalInfoMd, addedByAgentEmail, logoUrl, keywords }, referents, users }) => {
            var _a, _b;
            return ({
                "id": sillId,
                name,
                description,
                referencedSinceTime,
                updateTime,
                dereferencing,
                isStillInObservation,
                doRespectRgaa,
                isFromFrenchPublicService,
                isPresentInSupportContract,
                license,
                softwareType,
                catalogNumeriqueGouvFrId,
                versionMin,
                workshopUrls,
                testUrls,
                categories,
                generalInfoMd,
                addedByAgentEmail,
                logoUrl,
                keywords,
                "serviceProviders": (_a = serviceProvidersBySillId[sillId]) !== null && _a !== void 0 ? _a : [],
                "softwareExternalData": partialSoftwareBySillId[sillId].softwareExternalData,
                "similarExternalSoftwares": partialSoftwareBySillId[sillId].similarExternalSoftwares,
                "parentWikidataSoftware": partialSoftwareBySillId[sillId].parentWikidataSoftware,
                "comptoirDuLibreSoftware": comptoirDuLibreId === undefined
                    ? undefined
                    : (() => {
                        var _a, _b;
                        const cdlSoftware = cdlSoftwares.find(({ id }) => comptoirDuLibreId === id);
                        if (cdlSoftware === undefined) {
                            console.log([
                                `Comptoir du libre id: ${comptoirDuLibreId} does not match any software`,
                                `it might mean that the software has been recently added to comptoir du libre`,
                                `an their API is not yet up to date, ignoring`
                            ].join(" "));
                            return undefined;
                        }
                        const logoUrl = (_a = partialSoftwareBySillId[sillId].comptoirDuLibreSoftware) === null || _a === void 0 ? void 0 : _a.logoUrl;
                        const keywords = (_b = partialSoftwareBySillId[sillId].comptoirDuLibreSoftware) === null || _b === void 0 ? void 0 : _b.keywords;
                        return { ...cdlSoftware, logoUrl, keywords };
                    })(),
                "annuaireCnllServiceProviders": (_b = cnllPrestatairesSill
                    .find(({ sill_id }) => sill_id === sillId)) === null || _b === void 0 ? void 0 : _b.prestataires.map(({ nom, siren, url }) => ({
                    "name": nom,
                    siren,
                    url
                })),
                "latestVersion": partialSoftwareBySillId[sillId].latestVersion,
                users,
                referents,
                "instances": instanceRows
                    .filter(row => row.mainSoftwareSillId === sillId)
                    .map(({ id: instanceId, organization, targetAudience, publicUrl, addedByAgentEmail }) => ({
                    "id": instanceId,
                    organization,
                    targetAudience,
                    publicUrl,
                    "otherWikidataSoftwares": (() => {
                        const instance = partialSoftwareBySillId[sillId].instances.find(({ id }) => id === instanceId);
                        (0, assert_1.assert)(instance !== undefined);
                        return instance.otherWikidataSoftwares;
                    })(),
                    addedByAgentEmail
                }))
            });
        });
        return compiledData;
    };
    return { compileData };
}
exports.createCompileData = createCompileData;
//# sourceMappingURL=compileData.js.map