"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.comptoirDuLibreApi = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const ComptoirDuLibreApi_1 = require("../ports/ComptoirDuLibreApi");
const cheerio_1 = __importDefault(require("cheerio"));
const memoizee_1 = __importDefault(require("memoizee"));
const url = "https://comptoir-du-libre.org/public/export/comptoir-du-libre_export_v1.json";
exports.comptoirDuLibreApi = {
    "getComptoirDuLibre": (0, memoizee_1.default)(async () => {
        try {
            const res = await (0, node_fetch_1.default)(url);
            if (res.status !== 200) {
                throw new Error(`Failed to fetch ${url}`);
            }
            const text = await res.text();
            const json = JSON.parse(text);
            return ComptoirDuLibreApi_1.zComptoirDuLibre.parse(json);
        }
        catch (error) {
            console.error(`Failed to fetch or parse ${url}: ${String(error)}`);
            return {
                "date_of_export": Date.now().toString(),
                "number_of_software": 0,
                "softwares": []
            };
        }
    }, { "promise": true }),
    "getIconUrl": async ({ comptoirDuLibreId }) => {
        let imgSrc;
        try {
            const body = await (0, node_fetch_1.default)(`https://comptoir-du-libre.org/fr/softwares/${comptoirDuLibreId}`).then(r => r.text());
            const $ = cheerio_1.default.load(body);
            imgSrc = $(".size-logo-overview img").attr("src");
        }
        catch {
            return undefined;
        }
        if (imgSrc === undefined) {
            return undefined;
        }
        return `https://comptoir-du-libre.org/${imgSrc}`;
    },
    "getKeywords": async ({ comptoirDuLibreId }) => {
        let $;
        try {
            const body = await (0, node_fetch_1.default)(`https://comptoir-du-libre.org/fr/softwares/${comptoirDuLibreId}`).then(r => r.text());
            $ = cheerio_1.default.load(body);
        }
        catch {
            return [];
        }
        const keywords = [];
        let tagContainer;
        try {
            tagContainer = $(".tagsContainer");
        }
        catch {
            return [];
        }
        tagContainer.each(() => {
            let keyword;
            try {
                keyword = $(this).find(".tagUnit").text().trim();
            }
            catch {
                return;
            }
            keywords.push(keyword);
        });
        return keywords;
    }
};
//# sourceMappingURL=comptoirDuLibreApi.js.map