"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createGitDbApi = exports.compiledDataBranch = void 0;
const gitSsh_1 = require("../../tools/gitSsh");
const Deferred_1 = require("evt/tools/Deferred");
const CompileData_1 = require("../ports/CompileData");
const fs = __importStar(require("fs"));
const path_1 = require("path");
exports.compiledDataBranch = "compiled-data";
const compiledDataPrivateJsonRelativeFilePath = "compiledData_private.json";
const compiledDataPublicJsonRelativeFilePath = compiledDataPrivateJsonRelativeFilePath.replace(/_private.json$/, "_public.json");
const softwareJsonRelativeFilePath = "software.json";
const agentJsonRelativeFilePath = "agent.json";
const softwareReferentJsonRelativeFilePath = "softwareReferent.json";
const softwareUserJsonRelativeFilePath = "softwareUser.json";
const instanceJsonRelativeFilePath = "instance.json";
function createGitDbApi(params) {
    const { dataRepoSshUrl, sshPrivateKeyName, sshPrivateKey } = params;
    const dbApi = {
        "fetchCompiledData": () => {
            const dOut = new Deferred_1.Deferred();
            (0, gitSsh_1.gitSsh)({
                "sshUrl": dataRepoSshUrl,
                "shaish": exports.compiledDataBranch,
                sshPrivateKeyName,
                sshPrivateKey,
                "action": async ({ repoPath }) => {
                    const compiledData = JSON.parse((await fs.promises.readFile((0, path_1.join)(repoPath, compiledDataPrivateJsonRelativeFilePath))).toString("utf8"));
                    dOut.resolve(compiledData);
                    return { "doCommit": false };
                }
            }).catch(error => dOut.reject(error));
            return dOut.pr;
        },
        "fetchDb": () => {
            const dOut = new Deferred_1.Deferred();
            (0, gitSsh_1.gitSsh)({
                "sshUrl": dataRepoSshUrl,
                sshPrivateKeyName,
                sshPrivateKey,
                "action": async ({ repoPath }) => {
                    const [softwareRows, agentRows, softwareReferentRows, softwareUserRows, instanceRows] = await Promise.all([
                        softwareJsonRelativeFilePath,
                        agentJsonRelativeFilePath,
                        softwareReferentJsonRelativeFilePath,
                        softwareUserJsonRelativeFilePath,
                        instanceJsonRelativeFilePath
                    ]
                        .map(relativeFilePath => (0, path_1.join)(repoPath, relativeFilePath))
                        .map(filePath => fs.promises.readFile(filePath))).then(buffers => buffers.map(buffer => JSON.parse(buffer.toString("utf8"))));
                    dOut.resolve({
                        softwareRows,
                        agentRows,
                        softwareReferentRows,
                        softwareUserRows,
                        instanceRows
                    });
                    return { "doCommit": false };
                }
            }).catch(error => dOut.reject(error));
            return dOut.pr;
        },
        "updateDb": async ({ commitMessage, newDb }) => {
            await (0, gitSsh_1.gitSsh)({
                "sshUrl": dataRepoSshUrl,
                sshPrivateKeyName,
                sshPrivateKey,
                "action": async ({ repoPath }) => {
                    await Promise.all([
                        [softwareJsonRelativeFilePath, newDb.softwareRows],
                        [agentJsonRelativeFilePath, newDb.agentRows],
                        [softwareReferentJsonRelativeFilePath, newDb.softwareReferentRows],
                        [softwareUserJsonRelativeFilePath, newDb.softwareUserRows],
                        [instanceJsonRelativeFilePath, newDb.instanceRows]
                    ]
                        .map(([relativeFilePath, buffer]) => [(0, path_1.join)(repoPath, relativeFilePath), buffer])
                        .map(([filePath, rows]) => [filePath, JSON.stringify(rows, null, 4)])
                        .map(([filePath, rowsStr]) => [filePath, Buffer.from(rowsStr, "utf8")])
                        .map(([filePath, buffer]) => fs.promises.writeFile(filePath, buffer)));
                    return {
                        "doCommit": true,
                        "doAddAll": false,
                        "message": commitMessage
                    };
                }
            });
        },
        "updateCompiledData": async ({ newCompiledData, commitMessage }) => {
            await (0, gitSsh_1.gitSsh)({
                "sshUrl": dataRepoSshUrl,
                sshPrivateKeyName,
                sshPrivateKey,
                "shaish": exports.compiledDataBranch,
                "action": ({ repoPath }) => {
                    for (const [relativeJsonFilePath, data] of [
                        [compiledDataPrivateJsonRelativeFilePath, newCompiledData],
                        [compiledDataPublicJsonRelativeFilePath, (0, CompileData_1.compiledDataPrivateToPublic)(newCompiledData)]
                    ]) {
                        fs.writeFileSync((0, path_1.join)(repoPath, relativeJsonFilePath), Buffer.from(JSON.stringify(data, null, 2), "utf8"));
                    }
                    return Promise.resolve({
                        "doCommit": true,
                        "doAddAll": true,
                        "message": commitMessage
                    });
                }
            });
        }
    };
    const initializeDbApiCache = async () => {
        const start = Date.now();
        console.log("Starting dbApi cache initialization...");
        await Promise.all([
            (0, gitSsh_1.gitSsh)({
                "sshUrl": dataRepoSshUrl,
                "shaish": exports.compiledDataBranch,
                sshPrivateKeyName,
                sshPrivateKey,
                "action": async () => ({ "doCommit": false })
            }),
            (0, gitSsh_1.gitSsh)({
                "sshUrl": dataRepoSshUrl,
                sshPrivateKeyName,
                sshPrivateKey,
                "action": async () => ({ "doCommit": false })
            })
        ]);
        console.log(`dbApi cache initialization done in ${Date.now() - start}ms`);
    };
    return {
        dbApi,
        initializeDbApiCache
    };
}
exports.createGitDbApi = createGitDbApi;
//# sourceMappingURL=dbApi.js.map