"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getServiceProviders = void 0;
const memoizee_1 = __importDefault(require("memoizee"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const id_1 = require("tsafe/id");
const zod_1 = require("zod");
const assert_1 = require("tsafe/assert");
const zSillIdAndPrestataireFromApi = zod_1.z.object({
    "sill_id": zod_1.z.number(),
    "prestataires": zod_1.z.array(zod_1.z.object({
        "nom": zod_1.z.string(),
        "website": zod_1.z.string().optional(),
        "cdl_url": zod_1.z.string().optional(),
        "cnll_url": zod_1.z.string().optional(),
        "siren": zod_1.z.string().optional()
    }))
});
(0, assert_1.assert)();
const url = "https://code.gouv.fr/data/sill-prestataires.json";
exports.getServiceProviders = (0, memoizee_1.default)(async () => {
    try {
        const res = await (0, node_fetch_1.default)(url);
        if (res.status !== 200) {
            throw new Error(`Failed to fetch ${url}`);
        }
        const json = await res.json();
        const serviceProvidersFromApi = zod_1.z.array(zSillIdAndPrestataireFromApi).parse(json);
        return serviceProvidersFromApi.reduce((acc, { sill_id, prestataires }) => ({
            ...acc,
            [sill_id]: prestataires.map(prestataire => ({
                "name": prestataire.nom,
                "website": prestataire.website,
                "cdlUrl": prestataire.cdl_url,
                "cnllUrl": prestataire.cnll_url,
                "siren": prestataire.siren
            }))
        }), (0, id_1.id)({}));
    }
    catch (error) {
        console.error(`Failed to fetch or parse ${url}: ${String(error)}`);
        return {};
    }
}, { "promise": true });
//# sourceMappingURL=getServiceProviders.js.map