"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createGetSoftwareLatestVersion = void 0;
const getLatestSemVersionedTagFromSourceUrl_1 = require("../../tools/getLatestSemVersionedTagFromSourceUrl");
function createGetSoftwareLatestVersion(params) {
    const { githubPersonalAccessTokenForApiRateLimit } = params;
    const cacheQuick = new Map();
    const cacheLookEverywhere = new Map();
    async function getSoftwareLatestVersion(repoUrl, strategy) {
        {
            let cachedPr = undefined;
            cachedPr = cacheLookEverywhere.get(repoUrl);
            if (cachedPr !== undefined) {
                return cachedPr;
            }
            if (strategy === "quick") {
                cachedPr = cacheQuick.get(repoUrl);
                if (cachedPr !== undefined) {
                    return cachedPr;
                }
            }
        }
        const pr = (async () => {
            const resp = await (0, getLatestSemVersionedTagFromSourceUrl_1.getLatestSemVersionedTagFromSourceUrl)({
                githubPersonalAccessTokenForApiRateLimit,
                "sourceUrl": repoUrl,
                "isQuick": (() => {
                    switch (strategy) {
                        case "look everywhere":
                            return false;
                        case "quick":
                            return true;
                    }
                })()
            });
            if (resp === undefined) {
                return undefined;
            }
            const { version, publicationTime } = resp;
            return {
                "semVer": version,
                publicationTime
            };
        })();
        switch (strategy) {
            case "look everywhere":
                cacheLookEverywhere.set(repoUrl, pr);
                break;
            case "quick":
                cacheQuick.set(repoUrl, pr);
                break;
        }
        return pr;
    }
    getSoftwareLatestVersion.clear = (repoUrl) => {
        cacheLookEverywhere.delete(repoUrl);
        cacheQuick.delete(repoUrl);
    };
    return { getSoftwareLatestVersion };
}
exports.createGetSoftwareLatestVersion = createGetSoftwareLatestVersion;
//# sourceMappingURL=getSoftwareLatestVersion.js.map