"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.freeSoftwareLicensesWikidataIds = exports.WikidataFetchError = exports.getWikidataSoftware = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const cheerio = __importStar(require("cheerio"));
const assert_1 = require("tsafe/assert");
const memoizee_1 = __importDefault(require("memoizee"));
const noUndefined_1 = require("tsafe/noUndefined");
const allEquals_1 = require("evt/tools/reducers/allEquals");
const exclude_1 = require("tsafe/exclude");
const removeDuplicates_1 = require("evt/tools/reducers/removeDuplicates");
const same_1 = require("evt/tools/inDepth/same");
const reactless_1 = require("i18nifty/LocalizedString/reactless");
const id_1 = require("tsafe/id");
const GetSoftwareExternalData_1 = require("../ports/GetSoftwareExternalData");
const { resolveLocalizedString } = (0, reactless_1.createResolveLocalizedString)({
    "currentLanguage": (0, id_1.id)("en"),
    "fallbackLanguage": "en"
});
exports.getWikidataSoftware = (0, memoizee_1.default)(async (wikidataId) => {
    var _a, _b, _c;
    const { entity } = (_a = (await fetchEntity(wikidataId).catch(error => {
        if (error instanceof WikidataFetchError) {
            if (error.status === 404 || error.status === undefined) {
                return undefined;
            }
            throw error;
        }
    }))) !== null && _a !== void 0 ? _a : {};
    if (entity === undefined) {
        return undefined;
    }
    const { getClaimDataValue } = createGetClaimDataValue({ entity });
    const license = await (async () => {
        var _a, _b, _c;
        const licenseId = (_a = getClaimDataValue("P275")[0]) === null || _a === void 0 ? void 0 : _a.id;
        if (licenseId === undefined) {
            return undefined;
        }
        const { entity } = await fetchEntity(licenseId).catch(() => ({ "entity": undefined }));
        if (entity === undefined) {
            return undefined;
        }
        return { "label": (_c = (_b = entity.aliases.en) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.value, "id": licenseId };
    })();
    return {
        externalId: wikidataId,
        externalDataOrigin: "wikidata",
        "label": (_b = wikidataSingleLocalizedStringToLocalizedString(entity.labels)) !== null && _b !== void 0 ? _b : {
            "en": "No label"
        },
        "description": (_c = wikidataSingleLocalizedStringToLocalizedString(entity.descriptions)) !== null && _c !== void 0 ? _c : {
            "en": "No description"
        },
        "logoUrl": await (async () => {
            const value = getClaimDataValue("P154")[0];
            if (value === undefined) {
                return undefined;
            }
            const previewUrl = encodeURI(`https://www.wikidata.org/wiki/${wikidataId}#/media/File:${value}`);
            const raw = await (async function callee() {
                const res = await (0, node_fetch_1.default)(previewUrl).catch(() => undefined);
                if (res === undefined) {
                    return undefined;
                }
                // Too many requests
                if (res.status === 429) {
                    await new Promise(resolve => setTimeout(resolve, 1000));
                    return callee();
                }
                if (res.status !== 200) {
                    console.error(`Request to ${previewUrl} failed with error code ${res.status}`);
                    return undefined;
                }
                const raw = await res.text().catch(() => undefined);
                if (raw === undefined) {
                    return undefined;
                }
                return raw;
            })();
            if (raw === undefined) {
                return undefined;
            }
            const $ = cheerio.load(raw);
            const endOfHref = "File:" +
                encodeURIComponent(value)
                    .replace(/%2C/g, ",") //Preserve ','
                    .replace(/%20/g, "_") //Replace ' ' by '_'
                    .replace(/'/g, "%27"); //Replace ''' by '%27'
            const url = $(`a[href$="${endOfHref}"] img`).attr("src");
            (0, assert_1.assert)(url !== undefined, `Wikidata scrapper needs to be updated ${previewUrl} ${value}, endOfHref: ${endOfHref}`);
            return url;
        })(),
        "framaLibreId": getClaimDataValue("P4107")[0],
        ...(() => {
            const websiteUrl = getClaimDataValue("P856")[0];
            const sourceUrl = getClaimDataValue("P1324")[0];
            return {
                sourceUrl,
                "websiteUrl": sourceUrl !== websiteUrl ? websiteUrl : undefined
            };
        })(),
        "documentationUrl": getClaimDataValue("P2078")[0],
        "license": license === null || license === void 0 ? void 0 : license.label,
        "isLibreSoftware": license === undefined ? false : exports.freeSoftwareLicensesWikidataIds.includes(license.id),
        "developers": await Promise.all([
            ...getClaimDataValue("P50"),
            ...getClaimDataValue("P170"),
            ...getClaimDataValue("P172"),
            ...getClaimDataValue("P178")
        ].map(async ({ id }) => {
            const { entity } = await fetchEntity(id).catch(() => ({ "entity": undefined }));
            if (entity === undefined) {
                return undefined;
            }
            /*
            const { getClaimDataValue } = createGetClaimDataValue({
                entity,
            });

            const isHuman =
                getClaimDataValue<"wikibase-entityid">("P31").find(
                    ({ id }) => id === "Q5",
                ) !== undefined;

            if (!isHuman) {
                return undefined;
            }
            */
            const name = (() => {
                const { shortName } = (() => {
                    var _a;
                    const { getClaimDataValue } = createGetClaimDataValue({
                        entity
                    });
                    const shortName = (_a = getClaimDataValue("P1813")[0]) === null || _a === void 0 ? void 0 : _a.text;
                    return { shortName };
                })();
                if (shortName !== undefined) {
                    return shortName;
                }
                const label = wikidataSingleLocalizedStringToLocalizedString(entity.labels);
                if (label === undefined) {
                    return undefined;
                }
                return resolveLocalizedString(label);
            })();
            if (name === undefined) {
                return undefined;
            }
            return {
                name,
                "id": entity.id
            };
        })).then(developers => developers.filter((0, exclude_1.exclude)(undefined)).reduce(...(() => {
            const { removeDuplicates } = (0, removeDuplicates_1.removeDuplicatesFactory)({
                "areEquals": same_1.same
            });
            return removeDuplicates();
        })()))
    };
}, {
    "promise": true,
    "maxAge": 3 * 3600 * 1000
});
function wikidataSingleLocalizedStringToLocalizedString(wikidataSingleLocalizedString) {
    var _a;
    const localizedString = (0, noUndefined_1.noUndefined)(Object.fromEntries(GetSoftwareExternalData_1.languages.map(language => { var _a; return [language, (_a = wikidataSingleLocalizedString[language]) === null || _a === void 0 ? void 0 : _a.value]; })));
    if (Object.keys(localizedString).length === 0) {
        return (_a = wikidataSingleLocalizedString[Object.keys(wikidataSingleLocalizedString)[0]]) === null || _a === void 0 ? void 0 : _a.value;
    }
    if (Object.values(localizedString).reduce(...(0, allEquals_1.allEquals)())) {
        return localizedString[Object.keys(localizedString)[0]];
    }
    return localizedString;
}
class WikidataFetchError extends Error {
    constructor(status) {
        super(`Wikidata fetch error status: ${status}`);
        this.status = status;
        Object.setPrototypeOf(this, new.target.prototype);
    }
}
exports.WikidataFetchError = WikidataFetchError;
async function fetchEntity(wikidataId) {
    const res = await (0, node_fetch_1.default)(`https://www.wikidata.org/wiki/Special:EntityData/${wikidataId}.json`).catch(() => undefined);
    if (res === undefined) {
        throw new WikidataFetchError(undefined);
    }
    if (res.status === 429) {
        await new Promise(resolve => setTimeout(resolve, 100));
        return fetchEntity(wikidataId);
    }
    if (res.status === 404) {
        throw new WikidataFetchError(res.status);
    }
    const json = await res.json();
    const entity = Object.values(json["entities"])[0];
    return { entity };
}
function createGetClaimDataValue(params) {
    const { entity } = params;
    function getClaimDataValue(property) {
        const statementClaim = entity.claims[property];
        if (statementClaim === undefined) {
            return [];
        }
        return statementClaim
            .filter(x => x.rank !== "deprecated")
            .sort((a, b) => {
            const getWeight = (rank) => (rank === "preferred" ? 1 : 0);
            return getWeight(b.rank) - getWeight(a.rank);
        })
            .filter(x => x.mainsnak.snaktype === "value")
            .map(x => x.mainsnak.datavalue.value);
    }
    return { getClaimDataValue };
}
// Array of Free Software Licenses and their corresponding Wikidata IDs
exports.freeSoftwareLicensesWikidataIds = [
    // Apache License 2.0
    "Q309877",
    // BSD 2-Clause "Simplified" License
    "Q1507844",
    // BSD 3-Clause "New" or "Revised" License
    "Q1507824",
    // Eclipse Public License 2.0
    "Q5184255",
    // European Union Public License 1.2
    "Q65267454",
    // GNU Affero General Public License v3.0 only
    "Q1277061",
    // GNU Affero General Public License v3.0 or later
    "Q38926",
    // GNU General Public License v3.0 or later
    "Q2464622",
    // GNU Lesser General Public License v3.0 or later
    "Q39015",
    // MIT License
    "Q334661",
    // Mozilla Public License 2.0
    "Q334062",
    // CeCILL-B Free Software License Agreement
    "Q5099871",
    // CeCILL-C Free Software License Agreement
    "Q5099874",
    // CeCILL Free Software License Agreement v2.1
    "Q369616",
    // Academic Free License v3.0
    "Q467700",
    // Apache License 1.1
    "Q309884",
    // Apple Public Source License 2.0
    "Q466388",
    // Artistic License 2.0
    "Q6938433",
    // Boost Software License 1.0
    "Q333029",
    // Common Development and Distribution License 1.0
    "Q334209",
    // Common Public Attribution License 1.0
    "Q332884",
    // Common Public License 1.0
    "Q334393",
    // EU DataGrid Software License
    "Q5334061",
    // Eclipse Public License 1.0
    "Q334083",
    // Educational Community License v2.0
    "Q5358492",
    // Eiffel Forum License v2.0
    "Q465952",
    // European Union Public License 1.1
    "Q65267453",
    // GNU General Public License v2.0 only
    "Q7590",
    // GNU General Public License v2.0 or later
    "Q7553",
    // GNU General Public License v3.0 only
    "Q7571",
    // GNU Lesser General Public License v2.1 only
    "Q7547",
    // GNU Lesser General Public License v2.1 or later
    "Q30245",
    // GNU Lesser General Public License v3.0 only
    "Q7539",
    // GNU General Public License, version 3.0
    "Q10513445",
    // IBM Public License v1.0
    "Q467144",
    // ISC License
    "Q330779",
    // Intel Open Source License
    "Q607106",
    // Microsoft Public License
    "Q33057",
    // Microsoft Reciprocal License
    "Q33058",
    // Mozilla Public License 1.1
    "Q334395",
    // Open Software License 3.0
    "Q335473",
    // Python License 2.0
    "Q72189",
    // Q Public License 1.0
    "Q321678",
    // SIL Open Font License 1.1
    "Q55980",
    // Sun Public License v1.0
    "Q332889",
    // The Unlicense
    "Q6938435",
    // Universal Permissive License v1.0
    "Q107081891",
    // University of Illinois/NCSA Open Source License
    "Q667009",
    // Zlib License
    "Q207149",
    // Zope Public License 2.0
    "Q336266",
    // BSD licenses
    "Q191307"
];
//# sourceMappingURL=getWikidataSoftware.js.map