"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getWikidataSoftwareOptions = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const getWikidataSoftware_1 = require("./getWikidataSoftware");
const useAgent = "Socle interministériel de logiciels libres - Ap";
const getWikidataSoftwareOptions = async ({ queryString, language }) => {
    const results = (await (0, node_fetch_1.default)([
        "https://www.wikidata.org/w/api.php?action=wbsearchentities&format=json",
        `search=${encodeURIComponent(queryString)}`,
        `language=${language}`
    ].join("&"), {
        "headers": {
            "User-Agent": useAgent
        }
    }).then(response => response.json()));
    const arr = results.search.map(entry => {
        var _a, _b, _c;
        return ({
            "id": entry.id,
            "description": (_b = (_a = entry.display.description) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : "",
            "label": (_c = entry.label) !== null && _c !== void 0 ? _c : ""
        });
    });
    const licensesById = await getLicenses(arr.map(({ id }) => id));
    return arr.map(({ id, label, description }) => ({
        externalId: id,
        externalDataOrigin: "wikidata",
        label,
        description,
        "isLibreSoftware": (() => {
            const licenseId = licensesById[id];
            return licenseId === undefined ? false : getWikidataSoftware_1.freeSoftwareLicensesWikidataIds.includes(licenseId);
        })()
    }));
};
exports.getWikidataSoftwareOptions = getWikidataSoftwareOptions;
async function getLicenses(wikidataIds) {
    const propertyId = "P275"; // license
    const wikidataIdString = wikidataIds.map(id => "wd:" + id).join(" ");
    const query = `SELECT ?item ?itemLabel ?license ?licenseLabel WHERE {
        VALUES ?item { ${wikidataIdString} }
        ?item wdt:${propertyId} ?license.
        SERVICE wikibase:label { bd:serviceParam wikibase:language "en". }
    }`;
    const url = `https://query.wikidata.org/sparql?query=${encodeURIComponent(query)}&format=json`;
    const headers = { "User-Agent": useAgent };
    const response = await (0, node_fetch_1.default)(url, { headers });
    const data = await response.json();
    // Group the results by the Wikidata ID
    const groupedData = {};
    data.results.bindings.forEach((binding) => {
        const wikidataId = binding.item.value.split("/").pop();
        if (!groupedData[wikidataId]) {
            groupedData[wikidataId] = {
                "item": binding.item.value,
                "itemLabel": binding.itemLabel.value,
                "license": binding.license.value,
                "licenseLabel": binding.licenseLabel.value
            };
        }
    });
    return Object.fromEntries(Object.entries(groupedData).map(([wikidataId, { license }]) => [wikidataId, license.split("/").reverse()[0]]));
}
//# sourceMappingURL=getWikidataSoftwareOptions.js.map