"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchHalSoftwareById = exports.HalFetchError = exports.getHalSoftware = void 0;
const memoizee_1 = __importDefault(require("memoizee"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const halRawSoftware_1 = require("./halRawSoftware");
// HAL documentation is here : https://api.archives-ouvertes.fr/docs/search
exports.getHalSoftware = (0, memoizee_1.default)(async (halDocId) => {
    const halRawSoftware = await fetchHalSoftwareById(halDocId).catch(error => {
        if (!(error instanceof HalFetchError))
            throw error;
        if (error.status === 404 || error.status === undefined)
            return;
        throw error;
    });
    if (halRawSoftware === undefined)
        return;
    if (halRawSoftware.docType_s !== "SOFTWARE")
        return;
    return (0, halRawSoftware_1.rawHalSoftwareToSoftwareExternalData)(halRawSoftware);
}, {
    "promise": true,
    "maxAge": 3 * 3600 * 1000
});
class HalFetchError extends Error {
    constructor(status) {
        super(`Hal fetch error status: ${status}`);
        this.status = status;
        Object.setPrototypeOf(this, new.target.prototype);
    }
}
exports.HalFetchError = HalFetchError;
async function fetchHalSoftwareById(halDocid) {
    const res = await (0, node_fetch_1.default)(`https://api.archives-ouvertes.fr/search/?q=docid:${halDocid}&wt=json&fl=${halRawSoftware_1.halSoftwareFieldsToReturnAsString}&sort=docid%20asc`).catch(() => undefined);
    console.log("Hal response status : ", res === null || res === void 0 ? void 0 : res.status);
    if (res === undefined) {
        throw new HalFetchError(undefined);
    }
    if (res.status === 429) {
        await new Promise(resolve => setTimeout(resolve, 100));
        return fetchHalSoftwareById(halDocid);
    }
    if (res.status === 404) {
        throw new HalFetchError(res.status);
    }
    const json = await res.json();
    return json.response.docs[0];
}
exports.fetchHalSoftwareById = fetchHalSoftwareById;
//# sourceMappingURL=getHalSoftware.js.map