"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const getHalSoftware_1 = require("./getHalSoftware");
const getHalSoftwareOptions_1 = require("./getHalSoftwareOptions");
const expectToEqual = (actual, expected) => {
    (0, vitest_1.expect)(actual).toEqual(expected);
};
(0, vitest_1.describe)("HAL", () => {
    (0, vitest_1.describe)("getHalSoftware", () => {
        (0, vitest_1.it)("gets data from Hal and converts it to ExternalSoftware", async () => {
            // https://api.archives-ouvertes.fr/search/?q=docid:1510897&wt=json&fl=*&sort=docid%20asc
            const result = await (0, getHalSoftware_1.getHalSoftware)("1715545");
            expectToEqual(result, {
                "description": { "en": "-" },
                "developers": [
                    {
                        "id": "gruenpeter,-morane",
                        "name": "Gruenpeter, Morane"
                    }
                ],
                "documentationUrl": "https://inria.hal.science/hal-01715545",
                "externalId": "1715545",
                "framaLibreId": "",
                "isLibreSoftware": true,
                "label": {
                    "en": "Battleship exercise",
                    "fr": "Battleship exercise"
                },
                "license": "MIT License",
                "logoUrl": "",
                "externalDataOrigin": "HAL",
                "sourceUrl": "https://github.com/moranegg/Battleship",
                "websiteUrl": "https://inria.hal.science/hal-01715545"
            });
        });
    });
    (0, vitest_1.describe)("getHalSoftwareOption", () => {
        (0, vitest_1.it)("gets data from Hal and converts it to ExternalSoftwareOption, and returns the provided language", async () => {
            const enOptions = await (0, getHalSoftwareOptions_1.getHalSoftwareOptions)({ queryString: "multisensi", language: "en" });
            expectToEqual(enOptions, [
                {
                    "externalId": "2801278",
                    "label": "multisensi",
                    "description": "Functions to perform sensitivity analysis on a model with multivariate output.",
                    "isLibreSoftware": true,
                    "externalDataOrigin": "HAL"
                }
            ]);
            const frOptions = await (0, getHalSoftwareOptions_1.getHalSoftwareOptions)({ queryString: "multisensi", language: "fr" });
            expectToEqual(frOptions, [
                {
                    "externalId": "2801278",
                    "label": "multisensi : Analyse de sensibilité multivariée",
                    "description": "Functions to perform sensitivity analysis on a model with multivariate output.",
                    "isLibreSoftware": true,
                    "externalDataOrigin": "HAL"
                }
            ]);
        });
    });
});
//# sourceMappingURL=getHalSoftware.test.js.map