"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rawHalSoftwareToExternalOption = exports.rawHalSoftwareToSoftwareExternalData = exports.halSoftwareFieldsToReturnAsString = void 0;
const parseBibliographicFields_1 = require("./parseBibliographicFields");
const halSoftwareFieldsToReturn = [
    "en_abstract_s",
    "en_title_s",
    "fr_abstract_s",
    "fr_title_s",
    "docid",
    "uri_s",
    "openAccess_bool",
    "label_bibtex",
    "title_s",
    "abstract_s",
    "docType_s"
];
exports.halSoftwareFieldsToReturnAsString = halSoftwareFieldsToReturn.join(",");
const rawHalSoftwareToSoftwareExternalData = (halSoftware) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
    const bibliographicReferences = (0, parseBibliographicFields_1.parseBibliographicFields)(halSoftware.label_bibtex);
    const license = bibliographicReferences.license.join(", ");
    const developers = bibliographicReferences.author.map(author => ({
        id: author.toLowerCase().split(" ").join("-"),
        name: author
    }));
    return {
        externalId: halSoftware.docid,
        externalDataOrigin: "HAL",
        developers,
        label: {
            "en": (_d = (_b = (_a = halSoftware === null || halSoftware === void 0 ? void 0 : halSoftware.en_title_s) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : (_c = halSoftware === null || halSoftware === void 0 ? void 0 : halSoftware.title_s) === null || _c === void 0 ? void 0 : _c[0]) !== null && _d !== void 0 ? _d : "-",
            "fr": (_f = (_e = halSoftware === null || halSoftware === void 0 ? void 0 : halSoftware.fr_title_s) === null || _e === void 0 ? void 0 : _e[0]) !== null && _f !== void 0 ? _f : (_g = halSoftware.en_title_s) === null || _g === void 0 ? void 0 : _g[0]
        },
        description: {
            "en": (_l = (_j = (_h = halSoftware === null || halSoftware === void 0 ? void 0 : halSoftware.en_abstract_s) === null || _h === void 0 ? void 0 : _h[0]) !== null && _j !== void 0 ? _j : (_k = halSoftware.abstract_s) === null || _k === void 0 ? void 0 : _k[0]) !== null && _l !== void 0 ? _l : "-",
            "fr": (_o = (_m = halSoftware === null || halSoftware === void 0 ? void 0 : halSoftware.fr_abstract_s) === null || _m === void 0 ? void 0 : _m[0]) !== null && _o !== void 0 ? _o : (_p = halSoftware.en_abstract_s) === null || _p === void 0 ? void 0 : _p[0]
        },
        documentationUrl: halSoftware.uri_s,
        isLibreSoftware: halSoftware.openAccess_bool,
        license,
        sourceUrl: (_q = bibliographicReferences === null || bibliographicReferences === void 0 ? void 0 : bibliographicReferences.repository) === null || _q === void 0 ? void 0 : _q[0],
        websiteUrl: (_r = bibliographicReferences === null || bibliographicReferences === void 0 ? void 0 : bibliographicReferences.url) === null || _r === void 0 ? void 0 : _r[0],
        framaLibreId: "",
        logoUrl: ""
    };
};
exports.rawHalSoftwareToSoftwareExternalData = rawHalSoftwareToSoftwareExternalData;
const rawHalSoftwareToExternalOption = (language) => (halSoftware) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    const enLabel = (_d = (_b = (_a = halSoftware === null || halSoftware === void 0 ? void 0 : halSoftware.en_title_s) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : (_c = halSoftware === null || halSoftware === void 0 ? void 0 : halSoftware.title_s) === null || _c === void 0 ? void 0 : _c[0]) !== null && _d !== void 0 ? _d : "-";
    const labelByLang = {
        "en": enLabel,
        "fr": (_f = (_e = halSoftware === null || halSoftware === void 0 ? void 0 : halSoftware.fr_title_s) === null || _e === void 0 ? void 0 : _e[0]) !== null && _f !== void 0 ? _f : enLabel
    };
    const enDescription = (_k = (_h = (_g = halSoftware === null || halSoftware === void 0 ? void 0 : halSoftware.en_abstract_s) === null || _g === void 0 ? void 0 : _g[0]) !== null && _h !== void 0 ? _h : (_j = halSoftware.abstract_s) === null || _j === void 0 ? void 0 : _j[0]) !== null && _k !== void 0 ? _k : "-";
    const descriptionByLang = {
        "en": enDescription,
        "fr": (_m = (_l = halSoftware === null || halSoftware === void 0 ? void 0 : halSoftware.fr_abstract_s) === null || _l === void 0 ? void 0 : _l[0]) !== null && _m !== void 0 ? _m : enDescription
    };
    return {
        externalId: halSoftware.docid,
        label: labelByLang[language],
        description: descriptionByLang[language],
        isLibreSoftware: halSoftware.openAccess_bool,
        externalDataOrigin: "HAL"
    };
};
exports.rawHalSoftwareToExternalOption = rawHalSoftwareToExternalOption;
//# sourceMappingURL=halRawSoftware.js.map