"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createKeycloakUserApi = void 0;
const keycloakAdminApiClient_1 = require("../../tools/keycloakAdminApiClient");
const runExclusive = __importStar(require("run-exclusive"));
const memoizee_1 = __importDefault(require("memoizee"));
const maxAge = 5 * 60 * 1000;
function createKeycloakUserApi(params) {
    const { url, adminPassword, realm, organizationUserProfileAttributeName } = params;
    const keycloakAdminApiClient = (0, keycloakAdminApiClient_1.createKeycloakAdminApiClient)({
        url,
        adminPassword,
        realm
    });
    const groupRef = runExclusive.createGroupRef();
    const userApi = {
        "updateUserEmail": runExclusive.build(groupRef, ({ userId, email }) => keycloakAdminApiClient.updateUser({
            userId,
            "body": { email }
        })),
        "updateUserOrganization": runExclusive.build(groupRef, ({ userId, organization }) => keycloakAdminApiClient.updateUser({
            userId,
            "body": { "attributes": { [organizationUserProfileAttributeName]: organization } }
        })),
        "getAllowedEmailRegexp": (0, memoizee_1.default)(async () => {
            const attributes = await keycloakAdminApiClient.getUserProfileAttributes();
            let emailRegExpStr;
            try {
                emailRegExpStr = attributes.find(({ name }) => name === "email").validations.pattern
                    .pattern;
            }
            catch {
                throw new Error(`Can't extract RegExp from ${JSON.stringify(attributes)}`);
            }
            return emailRegExpStr;
        }, {
            "promise": true,
            maxAge,
            "preFetch": true
        }),
        "getAllOrganizations": (0, memoizee_1.default)(async () => {
            const organizations = new Set();
            let first = 0;
            // eslint-disable-next-line no-constant-condition
            while (true) {
                const max = 100;
                const users = await keycloakAdminApiClient.getUsers({
                    first,
                    max
                });
                users.forEach(user => {
                    let organization;
                    try {
                        organization = user.attributes[organizationUserProfileAttributeName][0];
                    }
                    catch {
                        console.log("Strange user: ", user);
                        return;
                    }
                    //NOTE: Hack, we had a bug so some organization are under
                    //"MESRI: Ministry of Higher Education, Research and Innovation" instead of "MESRI"
                    //(for example)
                    organization = organization.split(":")[0];
                    organizations.add(organization);
                });
                if (users.length < max) {
                    break;
                }
                first += max;
            }
            return Array.from(organizations);
        }, {
            "promise": true,
            maxAge,
            "preFetch": true
        }),
        "getUserCount": (0, memoizee_1.default)(async () => {
            let count = 0;
            let first = 0;
            // eslint-disable-next-line no-constant-condition
            while (true) {
                const max = 100;
                const users = await keycloakAdminApiClient.getUsers({
                    first,
                    max
                });
                count += users.length;
                if (users.length < max) {
                    break;
                }
                first += max;
            }
            return count;
        }, {
            "promise": true,
            maxAge,
            "preFetch": true
        })
    };
    const initializeUserApiCache = async () => {
        const start = Date.now();
        console.log("Starting userApi cache initialization...");
        await Promise.all(["getUserCount", "getAllOrganizations", "getAllowedEmailRegexp"].map(async function callee(methodName) {
            const f = userApi[methodName];
            await f();
            setInterval(f, maxAge - 10000);
        }));
        console.log(`userApi cache initialization done in ${Date.now() - start}ms`);
    };
    return { userApi, initializeUserApiCache };
}
exports.createKeycloakUserApi = createKeycloakUserApi;
//# sourceMappingURL=userApi.js.map