import { ServiceProvider } from "../usecases/readWriteSillData";
import type { Db } from "./DbApi";
import type { SoftwareExternalData } from "./GetSoftwareExternalData";
import type { ComptoirDuLibre } from "./ComptoirDuLibreApi";
export type CompileData = (params: {
    db: Db;
    getCachedSoftware: ((params: {
        sillSoftwareId: number;
    }) => CompileData.PartialSoftware | undefined) | undefined;
}) => Promise<CompiledData<"private">>;
export declare namespace CompileData {
    type PartialSoftware = Pick<CompiledData.Software<"private">, "softwareExternalData" | "latestVersion" | "similarExternalSoftwares" | "parentWikidataSoftware"> & {
        comptoirDuLibreSoftware: {
            id: number;
            logoUrl: string | undefined;
            keywords: string[] | undefined;
        } | undefined;
        instances: Pick<CompiledData.Instance, "id" | "otherWikidataSoftwares">[];
    };
}
export type CompiledData<T extends "private" | "public"> = CompiledData.Software<T>[];
export declare namespace CompiledData {
    type Software<T extends "private" | "public"> = T extends "private" ? Software.Private : Software.Public;
    namespace Software {
        type Common = Pick<Db.SoftwareRow, "id" | "name" | "description" | "referencedSinceTime" | "updateTime" | "dereferencing" | "isStillInObservation" | "doRespectRgaa" | "isFromFrenchPublicService" | "isPresentInSupportContract" | "license" | "softwareType" | "catalogNumeriqueGouvFrId" | "versionMin" | "workshopUrls" | "testUrls" | "categories" | "generalInfoMd" | "logoUrl" | "keywords" | "externalId" | "externalDataOrigin"> & {
            serviceProviders: ServiceProvider[];
            softwareExternalData: SoftwareExternalData | undefined;
            similarExternalSoftwares: Pick<SoftwareExternalData, "externalId" | "label" | "description" | "isLibreSoftware" | "externalDataOrigin">[];
            parentWikidataSoftware: Pick<SoftwareExternalData, "externalId" | "label" | "description"> | undefined;
            comptoirDuLibreSoftware: (ComptoirDuLibre.Software & {
                logoUrl: string | undefined;
                keywords: string[] | undefined;
            }) | undefined;
            annuaireCnllServiceProviders: {
                name: string;
                siren: string;
                url: string;
            }[] | undefined;
            latestVersion: {
                semVer: string;
                publicationTime: number;
            } | undefined;
        };
        type Public = Common & {
            userAndReferentCountByOrganization: Record<string, {
                userCount: number;
                referentCount: number;
            }>;
            hasExpertReferent: boolean;
            instances: Instance[];
        };
        type Private = Common & {
            addedByAgentEmail: string;
            users: (Pick<Db.AgentRow, "organization"> & Pick<Db.SoftwareUserRow, "os" | "serviceUrl" | "useCaseDescription" | "version">)[];
            referents: (Pick<Db.AgentRow, "email" | "organization"> & Pick<Db.SoftwareReferentRow, "isExpert" | "serviceUrl" | "useCaseDescription">)[];
            instances: (Instance & {
                addedByAgentEmail: string;
            })[];
        };
    }
    type Instance = {
        id: number;
        organization: string;
        targetAudience: string;
        publicUrl: string | undefined;
        otherWikidataSoftwares: Pick<SoftwareExternalData, "externalId" | "label" | "description">[];
    };
}
export declare function compiledDataPrivateToPublic(compiledData: CompiledData<"private">): CompiledData<"public">;
