"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.selectors = exports.protectedSelectors = void 0;
const redux_clean_architecture_1 = require("redux-clean-architecture");
const assert_1 = require("tsafe/assert");
const CompileData_1 = require("../../ports/CompileData");
const removeDuplicates_1 = require("evt/tools/reducers/removeDuplicates");
const exclude_1 = require("tsafe/exclude");
const id_1 = require("tsafe/id");
const state_1 = require("./state");
const sliceState = (state) => state[state_1.name];
const compiledData = (0, redux_clean_architecture_1.createSelector)(sliceState, state => state.compiledData);
const similarSoftwarePartition = (0, redux_clean_architecture_1.createSelector)(compiledData, (compiledData) => {
    const compiledSoftwareByWikidataId = {};
    compiledData.forEach(software => {
        if (software.softwareExternalData === undefined) {
            return;
        }
        compiledSoftwareByWikidataId[software.softwareExternalData.externalId] = software;
    });
    const compiledSoftwareByName = {};
    compiledData.forEach(software => {
        compiledSoftwareByName[software.name] = software;
    });
    function wikidataSoftwareToSimilarSoftware(externalSoftwareData) {
        const software = compiledSoftwareByWikidataId[externalSoftwareData.externalId];
        if (software === undefined) {
            return {
                "isInSill": false,
                "externalId": externalSoftwareData.externalId,
                "externalDataOrigin": externalSoftwareData.externalDataOrigin,
                "label": externalSoftwareData.label,
                "description": externalSoftwareData.description,
                "isLibreSoftware": externalSoftwareData.isLibreSoftware
            };
        }
        return {
            "isInSill": true,
            "softwareName": software.name,
            "softwareDescription": software.description
        };
    }
    const similarSoftwarePartition = [];
    compiledData.forEach(o => {
        const softwareAlreadySeen = new Set();
        for (const similarSoftwares of similarSoftwarePartition) {
            for (const similarSoftware of similarSoftwares) {
                if (!similarSoftware.isInSill) {
                    continue;
                }
                if (similarSoftware.softwareName === o.name) {
                    return;
                }
                softwareAlreadySeen.add(similarSoftware.softwareName);
            }
        }
        function getPartition(similarSoftware) {
            {
                const id = similarSoftware.isInSill ? similarSoftware.softwareName : similarSoftware.externalId;
                if (softwareAlreadySeen.has(id)) {
                    return [];
                }
                softwareAlreadySeen.add(id);
            }
            return (0, id_1.id)([
                similarSoftware,
                ...(() => {
                    if (!similarSoftware.isInSill) {
                        return [];
                    }
                    const software = compiledSoftwareByName[similarSoftware.softwareName];
                    (0, assert_1.assert)(software !== undefined);
                    return software.similarExternalSoftwares
                        .map(wikidataSoftware => getPartition(wikidataSoftwareToSimilarSoftware(wikidataSoftware)))
                        .flat();
                })(),
                ...compiledData
                    .map(software => {
                    const hasCurrentSimilarSoftwareInItsListOfSimilarSoftware = software.similarExternalSoftwares.find(externalSoftware => {
                        const similarSoftware_i = wikidataSoftwareToSimilarSoftware(externalSoftware);
                        if (similarSoftware.isInSill) {
                            return (similarSoftware_i.isInSill &&
                                similarSoftware_i.softwareName === similarSoftware.softwareName);
                        }
                        else {
                            return externalSoftware.externalId === similarSoftware.externalId;
                        }
                    }) !== undefined;
                    if (!hasCurrentSimilarSoftwareInItsListOfSimilarSoftware) {
                        return undefined;
                    }
                    return getPartition({
                        "isInSill": true,
                        "softwareName": software.name,
                        "softwareDescription": software.description
                    });
                })
                    .filter((0, exclude_1.exclude)(undefined))
                    .flat()
            ]);
        }
        similarSoftwarePartition.push(getPartition((0, id_1.id)({
            "isInSill": true,
            "softwareName": o.name,
            "softwareDescription": o.description
        })));
    });
    return similarSoftwarePartition;
});
const softwares = (0, redux_clean_architecture_1.createSelector)(compiledData, similarSoftwarePartition, (compiledData, similarSoftwarePartition) => {
    return compiledData.map((o) => {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y;
        return ({
            "serviceProviders": o.serviceProviders,
            "logoUrl": (_c = (_a = o.logoUrl) !== null && _a !== void 0 ? _a : (_b = o.softwareExternalData) === null || _b === void 0 ? void 0 : _b.logoUrl) !== null && _c !== void 0 ? _c : (_d = o.comptoirDuLibreSoftware) === null || _d === void 0 ? void 0 : _d.logoUrl,
            "softwareId": o.id,
            "softwareName": o.name,
            "softwareDescription": o.description,
            "latestVersion": o.latestVersion,
            "testUrl": (_e = o.testUrls[0]) === null || _e === void 0 ? void 0 : _e.url,
            "addedTime": o.referencedSinceTime,
            "updateTime": o.updateTime,
            "dereferencing": o.dereferencing,
            "categories": o.categories,
            "prerogatives": {
                "doRespectRgaa": o.doRespectRgaa,
                "isFromFrenchPublicServices": o.isFromFrenchPublicService,
                "isPresentInSupportContract": o.isPresentInSupportContract
            },
            "userAndReferentCountByOrganization": (() => {
                const out = {};
                o.referents.forEach(referent => {
                    var _a;
                    var _b;
                    const entry = ((_a = out[_b = referent.organization]) !== null && _a !== void 0 ? _a : (out[_b] = { "referentCount": 0, "userCount": 0 }));
                    entry.referentCount++;
                });
                o.users.forEach(user => {
                    var _a;
                    var _b;
                    const entry = ((_a = out[_b = user.organization]) !== null && _a !== void 0 ? _a : (out[_b] = { "referentCount": 0, "userCount": 0 }));
                    entry.userCount++;
                });
                return out;
            })(),
            "authors": (_g = (_f = o.softwareExternalData) === null || _f === void 0 ? void 0 : _f.developers.map(developer => ({
                "authorName": developer.name,
                "authorUrl": `https://www.wikidata.org/wiki/${developer.id}`
            }))) !== null && _g !== void 0 ? _g : [],
            "officialWebsiteUrl": (_l = (_j = (_h = o.softwareExternalData) === null || _h === void 0 ? void 0 : _h.websiteUrl) !== null && _j !== void 0 ? _j : (_k = o.comptoirDuLibreSoftware) === null || _k === void 0 ? void 0 : _k.external_resources.website) !== null && _l !== void 0 ? _l : undefined,
            "codeRepositoryUrl": (_q = (_o = (_m = o.softwareExternalData) === null || _m === void 0 ? void 0 : _m.sourceUrl) !== null && _o !== void 0 ? _o : (_p = o.comptoirDuLibreSoftware) === null || _p === void 0 ? void 0 : _p.external_resources.repository) !== null && _q !== void 0 ? _q : undefined,
            "documentationUrl": (_r = o.softwareExternalData) === null || _r === void 0 ? void 0 : _r.documentationUrl,
            "versionMin": o.versionMin,
            "license": o.license,
            "comptoirDuLibreServiceProviderCount": (_t = (_s = o.comptoirDuLibreSoftware) === null || _s === void 0 ? void 0 : _s.providers.length) !== null && _t !== void 0 ? _t : 0,
            "annuaireCnllServiceProviders": o.annuaireCnllServiceProviders,
            "comptoirDuLibreId": (_u = o.comptoirDuLibreSoftware) === null || _u === void 0 ? void 0 : _u.id,
            "externalId": (_v = o.softwareExternalData) === null || _v === void 0 ? void 0 : _v.externalId,
            "externalDataOrigin": (_w = o.softwareExternalData) === null || _w === void 0 ? void 0 : _w.externalDataOrigin,
            "softwareType": o.softwareType,
            "parentWikidataSoftware": o.parentWikidataSoftware,
            "similarSoftwares": (() => {
                for (const similarSoftwares of similarSoftwarePartition) {
                    for (const similarSoftware of similarSoftwares) {
                        if (!similarSoftware.isInSill) {
                            continue;
                        }
                        if (similarSoftware.softwareName === o.name) {
                            return similarSoftwares.filter(item => item !== similarSoftware);
                        }
                    }
                }
                return [];
            })(),
            "keywords": [...o.keywords, ...((_y = (_x = o.comptoirDuLibreSoftware) === null || _x === void 0 ? void 0 : _x.keywords) !== null && _y !== void 0 ? _y : [])].reduce(...(0, removeDuplicates_1.removeDuplicates)((k1, k2) => k1.toLowerCase() === k2.toLowerCase()))
        });
    });
});
const instances = (0, redux_clean_architecture_1.createSelector)(compiledData, (compiledData) => compiledData
    .map(software => software.instances.map(instance => ({ ...instance, "mainSoftwareSillId": software.id })))
    .flat()
    .map(({ id, organization, targetAudience, publicUrl, otherWikidataSoftwares, addedByAgentEmail, mainSoftwareSillId }) => ({
    id,
    mainSoftwareSillId,
    organization,
    targetAudience,
    publicUrl,
    otherWikidataSoftwares,
    addedByAgentEmail
})));
const agents = (0, redux_clean_architecture_1.createSelector)(sliceState, state => state.db.agentRows.map((agentRow) => {
    const getSoftwareName = (softwareId) => {
        const row = state.db.softwareRows.find(row => row.id === softwareId);
        (0, assert_1.assert)(row !== undefined);
        return row.name;
    };
    return {
        "email": agentRow.email,
        "declarations": [
            ...state.db.softwareUserRows
                .filter(row => row.agentEmail === agentRow.email)
                .map((row) => ({
                "declarationType": "user",
                "usecaseDescription": row.useCaseDescription,
                "os": row.os,
                "version": row.version,
                "serviceUrl": row.serviceUrl,
                "softwareName": getSoftwareName(row.softwareId)
            })),
            ...state.db.softwareReferentRows
                .filter(row => row.agentEmail === agentRow.email)
                .map((row) => ({
                "declarationType": "referent",
                "isTechnicalExpert": row.isExpert,
                "usecaseDescription": row.useCaseDescription,
                "serviceUrl": row.serviceUrl,
                "softwareName": getSoftwareName(row.softwareId)
            }))
        ],
        "organization": agentRow.organization
    };
}));
const aboutAndIsPublicByAgentEmail = (0, redux_clean_architecture_1.createSelector)(sliceState, (state) => Object.fromEntries(state.db.agentRows.map(agentRow => [
    agentRow.email,
    {
        "isPublic": agentRow.isPublic,
        "about": agentRow.about
    }
])));
const referentCount = (0, redux_clean_architecture_1.createSelector)(agents, agents => agents
    .filter(agent => agent.declarations.find(declaration => declaration.declarationType === "referent") !== undefined)
    .map(agent => agent.email)
    .reduce(...(0, removeDuplicates_1.removeDuplicates)()).length);
const compiledDataPublicJson = (0, redux_clean_architecture_1.createSelector)(sliceState, state => {
    const { compiledData } = state;
    return JSON.stringify((0, CompileData_1.compiledDataPrivateToPublic)(compiledData), null, 2);
});
exports.protectedSelectors = {
    similarSoftwarePartition
};
exports.selectors = {
    softwares,
    instances,
    agents,
    referentCount,
    compiledDataPublicJson,
    aboutAndIsPublicByAgentEmail
};
//# sourceMappingURL=selectors.js.map