"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.thunks = exports.protectedThunks = void 0;
const structured_clone_1 = __importDefault(require("@ungap/structured-clone"));
const redux_clean_architecture_1 = require("redux-clean-architecture");
const async_mutex_1 = require("async-mutex");
const assert_1 = require("tsafe/assert");
const same_1 = require("evt/tools/inDepth/same");
const Deferred_1 = require("evt/tools/Deferred");
const suggestionAndAutoFill = __importStar(require("../suggestionAndAutoFill"));
const objectKeys_1 = require("tsafe/objectKeys");
const state_1 = require("./state");
const selectors_1 = require("./selectors");
const { getContext } = (0, redux_clean_architecture_1.createUsecaseContextApi)(() => ({
    "mutex": new async_mutex_1.Mutex()
}));
exports.protectedThunks = {
    "initialize": (params) => async (...args) => {
        const { doPerPerformPeriodicalCompilation } = params;
        const [dispatch, getState, rootContext] = args;
        const { dbApi, evtAction } = rootContext;
        const [db, compiledData] = await Promise.all([dbApi.fetchDb(), dbApi.fetchCompiledData()]);
        dispatch(state_1.actions.updated({
            db,
            compiledData
        }));
        periodical_compilation: {
            if (!doPerPerformPeriodicalCompilation) {
                console.log("Periodical compilation disabled");
                break periodical_compilation;
            }
            console.log("Periodical update enabled");
            dispatch(privateThunks.triggerNonIncrementalCompilation({ "triggerType": "initial" }));
            setInterval(async () => {
                try {
                    dispatch(privateThunks.triggerNonIncrementalCompilation({ "triggerType": "periodical" }));
                }
                catch (error) {
                    console.error(`Non incremental periodical compilation failed: ${String(error)}`);
                }
            }, 4 * 3600 * 1000 //4 hour
            );
        }
        evtAction
            .pipe(action => action.usecaseName === state_1.name && action.actionName === "updated")
            .toStateful()
            .attach(() => {
            setTimeout(() => {
                const start = Date.now();
                console.log("Starting cache refresh of readWriteSillData selectors");
                (0, objectKeys_1.objectKeys)(selectors_1.selectors).forEach(selectorName => selectors_1.selectors[selectorName](getState()));
                console.log(`Cache refresh of readWriteSillData selectors done in ${Date.now() - start}ms`);
            }, 500);
        });
    }
};
exports.thunks = {
    "manuallyTriggerNonIncrementalCompilation": () => async (...args) => {
        const [dispatch] = args;
        await dispatch(privateThunks.triggerNonIncrementalCompilation({ "triggerType": "manual" }));
    },
    "notifyPushOnMainBranch": (params) => async (...args) => {
        const { commitMessage } = params;
        const [dispatch, , { dbApi }] = args;
        await dispatch(privateThunks.transaction(async () => ({
            "newDb": await dbApi.fetchDb(),
            commitMessage
        })));
    },
    "createSoftware": (params) => async (...args) => {
        const [dispatch] = args;
        const { formData } = params;
        const agentRow = { ...params.agent };
        await dispatch(privateThunks.transaction(async (newDb) => {
            const { softwareRows, agentRows } = newDb;
            (0, assert_1.assert)(softwareRows.find(s => {
                const t = (name) => name.toLowerCase().replace(/ /g, "-");
                return t(s.name) === t(formData.softwareName);
            }) === undefined, "There is already a software with this name");
            const softwareId = newDb.softwareRows.map(({ id }) => id).reduce((prev, curr) => Math.max(prev, curr), 0) + 1;
            const now = Date.now();
            softwareRows.push({
                "id": softwareId,
                "name": formData.softwareName,
                "description": formData.softwareDescription,
                "referencedSinceTime": now,
                "updateTime": now,
                "dereferencing": undefined,
                "isStillInObservation": false,
                "parentSoftwareWikidataId": undefined,
                "doRespectRgaa": formData.doRespectRgaa,
                "isFromFrenchPublicService": formData.isFromFrenchPublicService,
                "isPresentInSupportContract": formData.isPresentInSupportContract,
                "similarSoftwareExternalDataIds": formData.similarSoftwareExternalDataIds,
                "externalId": formData.externalId,
                "comptoirDuLibreId": formData.comptoirDuLibreId,
                "license": formData.softwareLicense,
                "softwareType": formData.softwareType,
                "catalogNumeriqueGouvFrId": undefined,
                "versionMin": formData.softwareMinimalVersion,
                "workshopUrls": [],
                "testUrls": [],
                "categories": [],
                "generalInfoMd": undefined,
                "addedByAgentEmail": agentRow.email,
                "logoUrl": await dispatch(privateThunks.getStorableLogo({
                    "externalId": formData.externalId,
                    "logoUrlFromFormData": formData.softwareLogoUrl
                })),
                "keywords": formData.softwareKeywords
            });
            if (agentRows.find(({ email }) => email === agentRow.email) === undefined) {
                agentRows.push({
                    "email": agentRow.email,
                    "organization": agentRow.organization,
                    "about": undefined,
                    "isPublic": false
                });
            }
            return {
                newDb,
                "commitMessage": `Add software: ${formData.softwareName}`
            };
        }));
    },
    "updateSoftware": (params) => async (...args) => {
        const [dispatch, getState] = args;
        const { softwareSillId, formData, agent } = params;
        await dispatch(privateThunks.transaction(async (newDb) => {
            const { softwareRows, softwareReferentRows } = newDb;
            (0, assert_1.assert)(softwareReferentRows.find(({ agentEmail }) => agentEmail === agentEmail) !== undefined, "The user is not a referent of this software");
            const index = softwareRows.findIndex(softwareRow => softwareRow.id === softwareSillId);
            (0, assert_1.assert)(index !== -1, "The software does not exist");
            let commitMessage = `${softwareRows[index].name} updated by ${agent.email}`;
            {
                const { id, referencedSinceTime, dereferencing, isStillInObservation, parentSoftwareWikidataId, addedByAgentEmail, catalogNumeriqueGouvFrId, categories, generalInfoMd, testUrls, workshopUrls, logoUrl: logoUrlFromDb } = softwareRows[index];
                const { comptoirDuLibreId, isFromFrenchPublicService, isPresentInSupportContract, similarSoftwareExternalDataIds, softwareDescription, softwareLicense, softwareMinimalVersion, softwareName, softwareType, externalId, softwareLogoUrl: logoUrlFromFormData, softwareKeywords, doRespectRgaa, ...rest } = formData;
                (0, assert_1.assert)();
                softwareRows[index] = {
                    id,
                    referencedSinceTime,
                    "updateTime": Date.now(),
                    dereferencing,
                    isStillInObservation,
                    parentSoftwareWikidataId,
                    doRespectRgaa,
                    addedByAgentEmail,
                    catalogNumeriqueGouvFrId,
                    categories,
                    generalInfoMd,
                    testUrls,
                    workshopUrls,
                    comptoirDuLibreId,
                    isFromFrenchPublicService,
                    isPresentInSupportContract,
                    similarSoftwareExternalDataIds,
                    "description": softwareDescription,
                    "license": softwareLicense,
                    "versionMin": softwareMinimalVersion,
                    "name": softwareName,
                    "softwareType": softwareType,
                    externalId,
                    "logoUrl": (() => {
                        const state = getState()[state_1.name];
                        const software = state.compiledData.find(({ id }) => id === softwareSillId);
                        (0, assert_1.assert)(software !== undefined);
                        if (software.logoUrl === logoUrlFromDb) {
                            return logoUrlFromDb;
                        }
                        return logoUrlFromFormData;
                    })(),
                    "keywords": softwareKeywords
                };
            }
            return {
                commitMessage,
                newDb
            };
        }));
    },
    "createUserOrReferent": (params) => async (...args) => {
        const [dispatch] = args;
        const { formData, softwareName, agent } = params;
        await dispatch(privateThunks.transaction(async (newDb) => {
            const { agentRows, softwareReferentRows, softwareUserRows, softwareRows } = newDb;
            const softwareRow = softwareRows.find(row => row.name === softwareName);
            (0, assert_1.assert)(softwareRow !== undefined, "Software not in SILL");
            if (agentRows.find(row => row.email === agent.email) === undefined) {
                agentRows.push({
                    "email": agent.email,
                    "organization": agent.organization,
                    "about": undefined,
                    "isPublic": false
                });
            }
            switch (formData.declarationType) {
                case "referent":
                    {
                        (0, assert_1.assert)(softwareReferentRows.find(row => row.softwareId === softwareRow.id && row.agentEmail === agent.email) === undefined, "Agent already referent of this software");
                        softwareReferentRows.push({
                            "softwareId": softwareRow.id,
                            "agentEmail": agent.email,
                            "isExpert": formData.isTechnicalExpert,
                            "serviceUrl": formData.serviceUrl,
                            "useCaseDescription": formData.usecaseDescription
                        });
                    }
                    break;
                case "user":
                    {
                        (0, assert_1.assert)(softwareUserRows.find(row => row.softwareId === softwareRow.id && row.agentEmail === agent.email) === undefined, "Agent already declared as user of this software");
                        softwareUserRows.push({
                            "softwareId": softwareRow.id,
                            "agentEmail": agent.email,
                            "os": formData.os,
                            "serviceUrl": formData.serviceUrl,
                            "useCaseDescription": formData.usecaseDescription,
                            "version": formData.version
                        });
                    }
                    break;
            }
            return {
                newDb,
                "commitMessage": `Add ${agent.email} as ${formData.declarationType} of ${softwareName}`
            };
        }));
    },
    "removeUserOrReferent": (params) => async (...args) => {
        const [dispatch] = args;
        const { softwareName, declarationType, agentEmail } = params;
        await dispatch(privateThunks.transaction(async (newDb) => {
            const { agentRows, softwareReferentRows, softwareUserRows, softwareRows, instanceRows } = newDb;
            const softwareRow = softwareRows.find(row => row.name === softwareName);
            (0, assert_1.assert)(softwareRow !== undefined, `There is no ${softwareName} in SILL`);
            const softwareDeclarationRows = (() => {
                switch (declarationType) {
                    case "referent":
                        return softwareReferentRows;
                    case "user":
                        return softwareUserRows;
                }
            })();
            const softwareDeclarationRow = softwareDeclarationRows.find(row => row.agentEmail === agentEmail && row.softwareId === softwareRow.id);
            (0, assert_1.assert)(softwareDeclarationRow !== undefined, `There is no ${agentEmail} as ${declarationType} of ${softwareName}`);
            softwareDeclarationRows.splice(softwareDeclarationRows.indexOf(softwareDeclarationRow), 1);
            remove_agent_if_no_longer_referenced_anywhere: {
                if (softwareReferentRows.find(row => row.agentEmail === agentEmail) !== undefined) {
                    break remove_agent_if_no_longer_referenced_anywhere;
                }
                if (softwareUserRows.find(row => row.agentEmail === agentEmail) !== undefined) {
                    break remove_agent_if_no_longer_referenced_anywhere;
                }
                if (softwareRows.find(row => row.addedByAgentEmail === agentEmail) !== undefined) {
                    break remove_agent_if_no_longer_referenced_anywhere;
                }
                if (instanceRows.find(row => row.addedByAgentEmail === agentEmail) !== undefined) {
                    break remove_agent_if_no_longer_referenced_anywhere;
                }
                const agentRow = agentRows.find(row => row.email === agentEmail);
                (0, assert_1.assert)(agentRow !== undefined, `There is no ${agentEmail} in the database`);
                agentRows.splice(agentRows.indexOf(agentRow), 1);
            }
            return {
                newDb,
                "commitMessage": `Remove ${agentEmail} as ${declarationType} of ${softwareName}`
            };
        }));
    },
    "createInstance": (params) => async (...args) => {
        const { agent, formData } = params;
        const [dispatch] = args;
        const dInstanceId = new Deferred_1.Deferred();
        await dispatch(privateThunks.transaction(async (newDb) => {
            const { instanceRows, softwareRows } = newDb;
            {
                const fmtUrl = (url) => url === undefined ? {} : url.toLowerCase().replace(/\/$/, "");
                (0, assert_1.assert)(instanceRows.find(row => row.mainSoftwareSillId === formData.mainSoftwareSillId &&
                    fmtUrl(row.publicUrl) === fmtUrl(formData.publicUrl)) === undefined, "This instance is already referenced");
            }
            const softwareRow = softwareRows.find(row => row.id === formData.mainSoftwareSillId);
            (0, assert_1.assert)(softwareRow !== undefined, "Can't create instance, software not in SILL");
            const instanceId = instanceRows.map(({ id }) => id).reduce((prev, curr) => Math.max(prev, curr), 0) + 1;
            dInstanceId.resolve({ instanceId });
            const now = Date.now();
            instanceRows.push({
                "id": instanceId,
                "addedByAgentEmail": agent.email,
                "organization": formData.organization,
                "mainSoftwareSillId": formData.mainSoftwareSillId,
                "otherSoftwareWikidataIds": formData.otherSoftwareWikidataIds,
                "publicUrl": formData.publicUrl,
                "targetAudience": formData.targetAudience,
                "referencedSinceTime": now,
                "updateTime": now
            });
            return {
                newDb,
                "commitMessage": `Adding ${softwareRow.name} instance: ${formData.publicUrl}`
            };
        }));
        return dInstanceId.pr;
    },
    "updateInstance": (params) => async (...args) => {
        const { instanceId, formData, agentEmail } = params;
        const [dispatch] = args;
        await dispatch(privateThunks.transaction(async (newDb) => {
            const { instanceRows } = newDb;
            const index = instanceRows.findIndex(row => row.id === instanceId);
            (0, assert_1.assert)(index !== -1, "Can't update instance, it doesn't exist");
            const { mainSoftwareSillId, organization, otherSoftwareWikidataIds, publicUrl, targetAudience, ...rest } = formData;
            (0, assert_1.assert)();
            const { id, addedByAgentEmail, referencedSinceTime } = instanceRows[index];
            instanceRows[index] = {
                id,
                addedByAgentEmail,
                mainSoftwareSillId,
                organization,
                otherSoftwareWikidataIds,
                publicUrl,
                targetAudience,
                referencedSinceTime,
                "updateTime": Date.now()
            };
            return {
                newDb,
                "commitMessage": `Instance ${formData.publicUrl} updated by ${agentEmail}`
            };
        }));
    },
    "changeAgentOrganization": (params) => async (...args) => {
        const [dispatch, , rootContext] = args;
        const { userApi } = rootContext;
        const { userId, email, newOrganization } = params;
        await dispatch(privateThunks.transaction(async (newDb) => {
            const { agentRows } = newDb;
            const agentRow = agentRows.find(row => row.email === email);
            if (agentRow === undefined) {
                return;
            }
            const { organization: oldOrganization } = agentRow;
            agentRow.organization = newOrganization;
            return {
                "result": undefined,
                newDb,
                "commitMessage": `Update ${email} organization from ${oldOrganization} to ${newOrganization}`
            };
        }));
        await userApi.updateUserOrganization({
            "organization": newOrganization,
            userId
        });
    },
    "updateUserEmail": (params) => async (...args) => {
        const [dispatch, , rootContext] = args;
        const { userApi } = rootContext;
        const { userId, email, newEmail } = params;
        await dispatch(privateThunks.transaction(async (newDb) => {
            const { agentRows, softwareReferentRows, softwareUserRows, softwareRows } = newDb;
            const agent = agentRows.find(row => row.email === email);
            if (agent === undefined) {
                return;
            }
            agent.email = newEmail;
            softwareReferentRows
                .filter(({ agentEmail }) => agentEmail === email)
                .forEach(softwareReferentRow => (softwareReferentRow.agentEmail = newEmail));
            softwareUserRows
                .filter(({ agentEmail }) => agentEmail === email)
                .forEach(softwareUserRow => (softwareUserRow.agentEmail = newEmail));
            softwareRows.filter(({ addedByAgentEmail }) => addedByAgentEmail === newEmail);
            return {
                "commitMessage": `Updating referent email from ${email} to ${newEmail}`,
                newDb
            };
        }));
        await userApi.updateUserEmail({
            "email": newEmail,
            userId
        });
    },
    "getAgentIsPublic": (params) => (...args) => {
        var _a;
        const [, getState] = args;
        const { email } = params;
        const { isPublic = false } = (_a = selectors_1.selectors.aboutAndIsPublicByAgentEmail(getState())[email]) !== null && _a !== void 0 ? _a : {};
        return isPublic;
    },
    "getAgent": (params) => async (...args) => {
        var _a, _b, _c;
        const [, getState] = args;
        const { email } = params;
        const state = getState();
        const { agent } = (() => {
            const agents = selectors_1.selectors.agents(state);
            const agent = agents.find(agent => agent.email === email);
            return { agent };
        })();
        const { isPublic = false, about } = (_a = selectors_1.selectors.aboutAndIsPublicByAgentEmail(state)[email]) !== null && _a !== void 0 ? _a : {};
        return {
            "email": email,
            "organization": (_b = agent === null || agent === void 0 ? void 0 : agent.organization) !== null && _b !== void 0 ? _b : "",
            "declarations": (_c = agent === null || agent === void 0 ? void 0 : agent.declarations) !== null && _c !== void 0 ? _c : [],
            isPublic,
            about
        };
    },
    "updateIsAgentProfilePublic": (params) => async (...args) => {
        const [dispatch] = args;
        const { agent: { email, organization }, isPublic } = params;
        await dispatch(privateThunks.transaction(async (newDb) => {
            const { agentRows } = newDb;
            let agentRow = agentRows.find(agentRow => agentRow.email === email);
            if (agentRow === undefined) {
                agentRow = {
                    email,
                    organization,
                    "isPublic": false,
                    "about": undefined
                };
                agentRows.push(agentRow);
            }
            agentRow.isPublic = isPublic;
            return {
                newDb,
                "commitMessage": `Making ${email} profile ${isPublic ? "public" : "private"}`
            };
        }));
    },
    "updateAgentAbout": (params) => async (...args) => {
        const [dispatch] = args;
        const { agent: { email, organization }, about } = params;
        await dispatch(privateThunks.transaction(async (newDb) => {
            const { agentRows } = newDb;
            let agentRow = agentRows.find(agentRow => agentRow.email === email);
            if (agentRow === undefined) {
                agentRow = {
                    email,
                    organization,
                    "isPublic": false,
                    "about": undefined
                };
                agentRows.push(agentRow);
            }
            agentRow.about = about;
            return {
                newDb,
                "commitMessage": `Updating ${email} about markdown text`
            };
        }));
    },
    "unreferenceSoftware": (params) => async (...args) => {
        const [dispatch] = args;
        const { softwareName, reason } = params;
        await dispatch(privateThunks.transaction(async (newDb) => {
            const { softwareRows } = newDb;
            const softwareRow = softwareRows.find(softwareRow => softwareRow.name === softwareName);
            (0, assert_1.assert)(softwareRow !== undefined, `There is no ${softwareName} in the database`);
            softwareRow.dereferencing = {
                "time": Date.now(),
                reason,
                "lastRecommendedVersion": softwareRow.versionMin
            };
            return {
                newDb,
                "commitMessage": `Dereferencing ${softwareName} because ${reason}`
            };
        }));
    }
};
const privateThunks = {
    "transaction": (asyncReducer) => async (...args) => {
        const [dispatch, getState, rootContext] = args;
        const { compileData, dbApi } = rootContext;
        const { mutex } = getContext(rootContext);
        const dLocalStateUpdated = new Deferred_1.Deferred();
        mutex
            .runExclusive(async () => {
            let newDb = (0, structured_clone_1.default)(getState()[state_1.name].db);
            const reducerReturnValue = await asyncReducer(newDb);
            if (reducerReturnValue === undefined) {
                return;
            }
            const { commitMessage, newDb: newDbReturnedByReducer } = reducerReturnValue;
            if (newDbReturnedByReducer !== undefined) {
                newDb = newDbReturnedByReducer;
            }
            const state = getState()[state_1.name];
            if ((0, same_1.same)(newDb, state.db)) {
                return;
            }
            //NOTE: It's important to call compileData first as it may crash
            //and if it does it mean that if we have committed we'll end up with
            //inconsistent state.
            const newCompiledData = await compileData({
                "db": newDb,
                "getCachedSoftware": ({ sillSoftwareId }) => state.compiledData.find(({ id }) => id === sillSoftwareId)
            });
            dispatch(state_1.actions.updated({
                "db": newDb,
                "compiledData": newCompiledData
            }));
            dLocalStateUpdated.resolve();
            try {
                await Promise.all([
                    dbApi.updateDb({ newDb, commitMessage }),
                    dbApi.updateCompiledData({ newCompiledData, commitMessage })
                ]);
            }
            catch (error) {
                console.error(`Error while updating the DB, this is fatal, terminating the process now`, String(error));
                process.exit(1);
            }
        })
            .catch(error => dLocalStateUpdated.reject(error));
        await dLocalStateUpdated.pr;
    },
    "triggerNonIncrementalCompilation": (params) => async (...args) => {
        console.log("Starting non incremental compilation");
        const { triggerType } = params;
        const [dispatch, getState, rootContext] = args;
        const { dbApi, compileData } = rootContext;
        const { mutex } = getContext(rootContext);
        const dbBefore = (0, structured_clone_1.default)(getState()[state_1.name].db);
        console.log("Non incremental compilation started");
        const newCompiledData = await compileData({
            "db": dbBefore,
            "getCachedSoftware": undefined
        });
        const wasCanceled = await mutex.runExclusive(async () => {
            const { db } = getState()[state_1.name];
            if (!(0, same_1.same)(dbBefore, db)) {
                //While we where re compiling there was some other transaction,
                //Re-scheduling.
                console.log("Re-scheduling non incremental compilation, db has changed (probably due to a concurrent transaction)");
                return true;
            }
            await dbApi.updateCompiledData({
                newCompiledData,
                "commitMessage": (() => {
                    switch (triggerType) {
                        case "initial":
                            return "Some data have changed while the backend was down";
                        case "periodical":
                            return "Periodical update: Some Wikidata or other 3rd party source data have changed";
                        case "manual":
                            return "Manual trigger: Some data have changed since last compilation";
                    }
                })()
            });
            dispatch(state_1.actions.updated({
                db,
                "compiledData": newCompiledData
            }));
            return false;
        });
        if (wasCanceled) {
            console.log("Data have changed, re-scheduling non incremental compilation");
            await dispatch(privateThunks.triggerNonIncrementalCompilation(params));
        }
        console.log("Done with non incremental compilation");
    },
    /** Functions that returns logoUrlFromFormData if it's not the same as the one from the automatic suggestions */
    "getStorableLogo": (params) => async (...args) => {
        const { logoUrlFromFormData, externalId } = params;
        const [dispatch] = args;
        if (logoUrlFromFormData === undefined) {
            return undefined;
        }
        if (externalId === undefined) {
            return logoUrlFromFormData;
        }
        const softwareFormAutoFillData = await dispatch(suggestionAndAutoFill.thunks.getSoftwareFormAutoFillDataFromExternalAndOtherSources({
            externalId
        }));
        if (softwareFormAutoFillData.softwareLogoUrl === logoUrlFromFormData) {
            return undefined;
        }
        return logoUrlFromFormData;
    }
};
//# sourceMappingURL=thunks.js.map