import type { ExternalDataOrigin, SoftwareExternalData } from "../../ports/GetSoftwareExternalData";
export type ServiceProvider = {
    name: string;
    website?: string;
    cdlUrl?: string;
    cnllUrl?: string;
    siren?: string;
};
export type Software = {
    logoUrl: string | undefined;
    softwareId: number;
    softwareName: string;
    softwareDescription: string;
    serviceProviders: ServiceProvider[];
    latestVersion: {
        semVer: string;
        publicationTime: number;
    } | undefined;
    testUrl: string | undefined;
    addedTime: number;
    updateTime: number;
    dereferencing: {
        reason?: string;
        time: number;
        lastRecommendedVersion?: string;
    } | undefined;
    categories: string[];
    prerogatives: Prerogatives;
    userAndReferentCountByOrganization: Record<string, {
        userCount: number;
        referentCount: number;
    }>;
    authors: {
        authorName: string;
        authorUrl: string;
    }[];
    officialWebsiteUrl: string | undefined;
    codeRepositoryUrl: string | undefined;
    documentationUrl: string | undefined;
    versionMin: string;
    license: string;
    comptoirDuLibreServiceProviderCount: number;
    annuaireCnllServiceProviders: {
        name: string;
        siren: string;
        url: string;
    }[] | undefined;
    comptoirDuLibreId: number | undefined;
    externalId: string | undefined;
    externalDataOrigin: ExternalDataOrigin | undefined;
    softwareType: SoftwareType;
    parentWikidataSoftware: Pick<SoftwareExternalData, "externalId" | "label" | "description"> | undefined;
    similarSoftwares: Software.SimilarSoftware[];
    keywords: string[];
};
export declare namespace Software {
    type SimilarSoftware = SimilarSoftware.ExternalSoftwareData | SimilarSoftware.Sill;
    namespace SimilarSoftware {
        type ExternalSoftwareData = {
            isInSill: false;
        } & Pick<SoftwareExternalData, "externalId" | "label" | "description" | "isLibreSoftware" | "externalDataOrigin">;
        type Sill = {
            isInSill: true;
            softwareName: string;
            softwareDescription: string;
        };
    }
}
export type Agent = {
    email: string | undefined;
    organization: string;
    declarations: (DeclarationFormData & {
        softwareName: string;
    })[];
};
export type Instance = {
    id: number;
    mainSoftwareSillId: number;
    organization: string;
    targetAudience: string;
    publicUrl: string | undefined;
    otherWikidataSoftwares: Pick<SoftwareExternalData, "externalId" | "label" | "description">[];
};
export type SoftwareType = SoftwareType.Desktop | SoftwareType.CloudNative | SoftwareType.Stack;
export declare namespace SoftwareType {
    type Desktop = {
        type: "desktop/mobile";
        os: Record<Os, boolean>;
    };
    type CloudNative = {
        type: "cloud";
    };
    type Stack = {
        type: "stack";
    };
}
type Prerogatives = {
    isPresentInSupportContract: boolean;
    isFromFrenchPublicServices: boolean;
    doRespectRgaa: boolean | null;
};
export type Prerogative = keyof Prerogatives;
export type Os = "windows" | "linux" | "mac" | "android" | "ios";
export type SoftwareFormData = {
    softwareType: SoftwareType;
    externalId: string | undefined;
    comptoirDuLibreId: number | undefined;
    softwareName: string;
    softwareDescription: string;
    softwareLicense: string;
    softwareMinimalVersion: string;
    isPresentInSupportContract: boolean;
    isFromFrenchPublicService: boolean;
    similarSoftwareExternalDataIds: string[];
    softwareLogoUrl: string | undefined;
    softwareKeywords: string[];
    doRespectRgaa: boolean | null;
};
export type DeclarationFormData = DeclarationFormData.User | DeclarationFormData.Referent;
export declare namespace DeclarationFormData {
    type User = {
        declarationType: "user";
        usecaseDescription: string;
        /** NOTE: undefined if the software is not of type desktop/mobile */
        os: Os | undefined;
        version: string;
        /** NOTE: Defined only when software is cloud */
        serviceUrl: string | undefined;
    };
    type Referent = {
        declarationType: "referent";
        isTechnicalExpert: boolean;
        usecaseDescription: string;
        /** NOTE: Can be not undefined only if cloud */
        serviceUrl: string | undefined;
    };
}
export type InstanceFormData = {
    mainSoftwareSillId: number;
    organization: string;
    targetAudience: string;
    publicUrl: string | undefined;
    otherSoftwareWikidataIds: string[];
};
export {};
