"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.thunks = void 0;
const redux_clean_architecture_1 = require("redux-clean-architecture");
const assert_1 = require("tsafe/assert");
const reactless_1 = require("i18nifty/LocalizedString/reactless");
const id_1 = require("tsafe/id");
const selectors_1 = require("./selectors");
exports.thunks = {
    "getSoftwareExternalDataOptionsWithPresenceInSill": (params) => async (...args) => {
        const { queryString, language } = params;
        const [, getState, { getSoftwareExternalDataOptions }] = args;
        const queryResults = await getSoftwareExternalDataOptions({ queryString, language });
        const sillWikidataIds = selectors_1.privateSelectors.sillWikidataIds(getState());
        return queryResults.map(({ externalId, description, label, isLibreSoftware, externalDataOrigin }) => ({
            "externalId": externalId,
            "description": description,
            "label": label,
            "isInSill": sillWikidataIds.includes(externalId),
            isLibreSoftware,
            "externalDataOrigin": externalDataOrigin
        }));
    },
    "getSoftwareFormAutoFillDataFromExternalAndOtherSources": (params) => async (...args) => {
        var _a, _b;
        const { externalId } = params;
        const [, , rootContext] = args;
        const { autoFillDataCache } = getContext(rootContext);
        {
            const cachedAutoFillData = autoFillDataCache[externalId];
            if (cachedAutoFillData !== undefined) {
                return cachedAutoFillData;
            }
        }
        const { getSoftwareLatestVersion, comptoirDuLibreApi, getSoftwareExternalData } = rootContext;
        const [softwareExternalData, comptoirDuLibre] = await Promise.all([
            getSoftwareExternalData(externalId),
            comptoirDuLibreApi.getComptoirDuLibre()
        ]);
        (0, assert_1.assert)(softwareExternalData !== undefined);
        const { label: externalSoftwareLabel } = softwareExternalData;
        const { comptoirDuLibreSoftware } = (() => {
            if (externalSoftwareLabel === undefined) {
                return { "comptoirDuLibreSoftware": undefined };
            }
            const comptoirDuLibreSoftware = comptoirDuLibre.softwares.find(software => {
                const format = (name) => name
                    .normalize("NFD")
                    .replace(/[\u0300-\u036f]/g, "")
                    .toLowerCase()
                    .replace(/ g/, "");
                const { resolveLocalizedString } = (0, reactless_1.createResolveLocalizedString)({
                    "currentLanguage": "en",
                    "fallbackLanguage": "en"
                });
                return format(software.name).includes(format(resolveLocalizedString(externalSoftwareLabel)).substring(0, 8));
            });
            return { comptoirDuLibreSoftware };
        })();
        const [comptoirDuLibreLogoUrl, comptoirDuLibreKeywords] = comptoirDuLibreSoftware === undefined
            ? [undefined, undefined]
            : await Promise.all([
                comptoirDuLibreApi.getIconUrl({ "comptoirDuLibreId": comptoirDuLibreSoftware.id }),
                comptoirDuLibreApi.getKeywords({ "comptoirDuLibreId": comptoirDuLibreSoftware.id })
            ]);
        const { resolveLocalizedString } = (0, reactless_1.createResolveLocalizedString)({
            "currentLanguage": "fr",
            "fallbackLanguage": "en"
        });
        const autoFillData = {
            "comptoirDuLibreId": comptoirDuLibreSoftware === null || comptoirDuLibreSoftware === void 0 ? void 0 : comptoirDuLibreSoftware.id,
            "softwareName": externalSoftwareLabel === undefined ? undefined : resolveLocalizedString(externalSoftwareLabel),
            "softwareDescription": softwareExternalData.description === undefined
                ? undefined
                : resolveLocalizedString(softwareExternalData.description),
            "softwareLicense": (_a = softwareExternalData.license) !== null && _a !== void 0 ? _a : comptoirDuLibreSoftware === null || comptoirDuLibreSoftware === void 0 ? void 0 : comptoirDuLibreSoftware.licence,
            "softwareMinimalVersion": await (async () => {
                var _a, _b;
                const repoUrl = (_b = (_a = softwareExternalData.sourceUrl) !== null && _a !== void 0 ? _a : comptoirDuLibreSoftware === null || comptoirDuLibreSoftware === void 0 ? void 0 : comptoirDuLibreSoftware.external_resources.repository) !== null && _b !== void 0 ? _b : undefined;
                return repoUrl === undefined
                    ? undefined
                    : getSoftwareLatestVersion(repoUrl, "quick").then(resp => resp === null || resp === void 0 ? void 0 : resp.semVer);
            })(),
            "softwareLogoUrl": (_b = softwareExternalData.logoUrl) !== null && _b !== void 0 ? _b : comptoirDuLibreLogoUrl,
            "keywords": comptoirDuLibreKeywords !== null && comptoirDuLibreKeywords !== void 0 ? comptoirDuLibreKeywords : []
        };
        autoFillDataCache[externalId] = autoFillData;
        setTimeout(() => {
            delete autoFillDataCache[externalId];
        }, 3 * 60 * 1000 /* 3 hours */);
        return autoFillData;
    }
};
const { getContext } = (0, redux_clean_architecture_1.createUsecaseContextApi)(() => ({
    "autoFillDataCache": (0, id_1.id)({})
}));
//# sourceMappingURL=thunks.js.map