import type { ReturnType } from "tsafe";
import type { Core, Context as CoreContext } from "../core";
import type { Context } from "./context";
import type { User } from "./user";
import type { KeycloakParams } from "../tools/createValidateKeycloakSignature";
import type { SoftwareFormData, DeclarationFormData, InstanceFormData } from "../core/usecases/readWriteSillData";
import superjson from "superjson";
import { type LocalizedString, ExternalDataOrigin } from "../core/ports/GetSoftwareExternalData";
export declare function createRouter(params: {
    core: Core;
    coreContext: CoreContext;
    keycloakParams: (KeycloakParams & {
        organizationUserProfileAttributeName: string;
    }) | undefined;
    jwtClaimByUserKey: Record<keyof User, string>;
    termsOfServiceUrl: LocalizedString;
    readmeUrl: LocalizedString;
    redirectUrl: string | undefined;
    externalSoftwareDataOrigin: ExternalDataOrigin;
}): {
    router: import("@trpc/server").CreateRouterInner<import("@trpc/server").RootConfig<{
        ctx: Context;
        meta: object;
        errorShape: import("@trpc/server").DefaultErrorShape;
        transformer: typeof superjson;
    }>, {
        getRedirectUrl: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, string | undefined>;
        getExternalSoftwareDataOrigin: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, "wikidata" | "HAL">;
        getApiVersion: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, string>;
        getOidcParams: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, {
            keycloakParams: {
                url: string;
                realm: string;
                clientId: string;
            } | undefined;
            jwtClaimByUserKey: Record<keyof User, string>;
        }>;
        getOrganizationUserProfileAttributeName: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, string>;
        getSoftwares: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, import("../core/usecases/readWriteSillData").Software[]>;
        getInstances: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, import("../core/usecases/readWriteSillData").Instance[]>;
        getExternalSoftwareOptions: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                queryString: string;
                language: "fr" | "en";
            };
            _input_out: {
                queryString: string;
                language: "fr" | "en";
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, {
            externalId: string;
            description: string;
            label: string;
            isInSill: boolean;
            isLibreSoftware: boolean;
            externalDataOrigin: ExternalDataOrigin;
        }[]>;
        getSoftwareFormAutoFillDataFromExternalSoftwareAndOtherSources: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                externalId: string;
            };
            _input_out: {
                externalId: string;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, {
            comptoirDuLibreId: number | undefined;
            softwareName: string | undefined;
            softwareDescription: string | undefined;
            softwareLicense: string | undefined;
            softwareMinimalVersion: string | undefined;
            softwareLogoUrl: string | undefined;
            keywords: string[];
        }>;
        createSoftware: import("@trpc/server").BuildProcedure<"mutation", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                formData: SoftwareFormData;
            };
            _input_out: {
                formData: SoftwareFormData;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, void>;
        updateSoftware: import("@trpc/server").BuildProcedure<"mutation", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                formData: SoftwareFormData;
                softwareSillId: number;
            };
            _input_out: {
                formData: SoftwareFormData;
                softwareSillId: number;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, void>;
        createUserOrReferent: import("@trpc/server").BuildProcedure<"mutation", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                softwareName: string;
                formData: DeclarationFormData.User | DeclarationFormData.Referent;
            };
            _input_out: {
                softwareName: string;
                formData: DeclarationFormData.User | DeclarationFormData.Referent;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, void>;
        removeUserOrReferent: import("@trpc/server").BuildProcedure<"mutation", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                softwareName: string;
                declarationType: "user" | "referent";
            };
            _input_out: {
                softwareName: string;
                declarationType: "user" | "referent";
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, void>;
        createInstance: import("@trpc/server").BuildProcedure<"mutation", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                formData: InstanceFormData;
            };
            _input_out: {
                formData: InstanceFormData;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, {
            instanceId: number;
        }>;
        updateInstance: import("@trpc/server").BuildProcedure<"mutation", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                formData: InstanceFormData;
                instanceId: number;
            };
            _input_out: {
                formData: InstanceFormData;
                instanceId: number;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, void>;
        getAgents: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, {
            agents: import("../core/usecases/readWriteSillData").Agent[];
        }>;
        updateIsAgentProfilePublic: import("@trpc/server").BuildProcedure<"mutation", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                isPublic: boolean;
            };
            _input_out: {
                isPublic: boolean;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, void>;
        updateAgentAbout: import("@trpc/server").BuildProcedure<"mutation", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                about?: string | undefined;
            };
            _input_out: {
                about?: string | undefined;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, void>;
        getIsAgentProfilePublic: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                email: string;
            };
            _input_out: {
                email: string;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, {
            isPublic: boolean;
        }>;
        getAgent: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                email: string;
            };
            _input_out: {
                email: string;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, {
            agent: import("../core/usecases/readWriteSillData").Agent & {
                isPublic: boolean;
                about: string | undefined;
            };
        }>;
        getAllowedEmailRegexp: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, string>;
        getAllOrganizations: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, string[]>;
        changeAgentOrganization: import("@trpc/server").BuildProcedure<"mutation", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                newOrganization: string;
            };
            _input_out: {
                newOrganization: string;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, void>;
        updateEmail: import("@trpc/server").BuildProcedure<"mutation", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                newEmail: string;
            };
            _input_out: {
                newEmail: string;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, void>;
        getRegisteredUserCount: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, number>;
        getTotalReferentCount: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, {
            referentCount: number;
        }>;
        getTermsOfServiceUrl: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: typeof import("@trpc/server").unsetMarker;
            _input_out: typeof import("@trpc/server").unsetMarker;
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, string | Partial<Record<"fr" | "en", string>>>;
        getMarkdown: import("@trpc/server").BuildProcedure<"query", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                name: "readme" | "termsOfService";
                language: "fr" | "en";
            };
            _input_out: {
                name: "readme" | "termsOfService";
                language: "fr" | "en";
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, string>;
        unreferenceSoftware: import("@trpc/server").BuildProcedure<"mutation", {
            _config: import("@trpc/server").RootConfig<{
                ctx: Context;
                meta: object;
                errorShape: import("@trpc/server").DefaultErrorShape;
                transformer: typeof superjson;
            }>;
            _meta: object;
            _ctx_out: import("@trpc/server").Overwrite<Context, Context>;
            _input_in: {
                softwareName: string;
                reason: string;
            };
            _input_out: {
                softwareName: string;
                reason: string;
            };
            _output_in: typeof import("@trpc/server").unsetMarker;
            _output_out: typeof import("@trpc/server").unsetMarker;
        }, void>;
    }>;
};
export type TrpcRouter = ReturnType<typeof createRouter>["router"];
