"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRouter = void 0;
const server_1 = require("@trpc/server");
const assert_1 = require("tsafe/assert");
const zod_1 = require("zod");
const fs = __importStar(require("fs"));
const path_1 = require("path");
const getProjectRoot_1 = require("../tools/getProjectRoot");
const node_fetch_1 = __importDefault(require("node-fetch"));
const memoizee_1 = __importDefault(require("memoizee"));
const server_2 = require("@trpc/server");
const superjson_1 = __importDefault(require("superjson"));
const GetSoftwareExternalData_1 = require("../core/ports/GetSoftwareExternalData");
const reactless_1 = require("i18nifty/LocalizedString/reactless");
function createRouter(params) {
    const { core, coreContext, keycloakParams, jwtClaimByUserKey, termsOfServiceUrl, readmeUrl, redirectUrl, externalSoftwareDataOrigin } = params;
    const t = server_2.initTRPC.context().create({
        "transformer": superjson_1.default
    });
    const loggedProcedure = t.procedure.use(t.middleware(async (opts) => {
        const start = Date.now();
        const result = await opts.next();
        const durationMs = Date.now() - start;
        const meta = { "path": opts.path, "type": opts.type, durationMs };
        result.ok ? console.log("OK request timing:", meta) : console.error("Non-OK request timing", meta);
        return result;
    }));
    const router = t.router({
        "getRedirectUrl": loggedProcedure.query(() => redirectUrl),
        "getExternalSoftwareDataOrigin": loggedProcedure.query(() => externalSoftwareDataOrigin),
        "getApiVersion": loggedProcedure.query((() => {
            const out = JSON.parse(fs.readFileSync((0, path_1.join)((0, getProjectRoot_1.getProjectRoot)(), "package.json")).toString("utf8"))["version"];
            return () => out;
        })()),
        "getOidcParams": loggedProcedure.query((() => {
            const out = {
                "keycloakParams": (() => {
                    if (keycloakParams === undefined) {
                        return undefined;
                    }
                    const { url, realm, clientId } = keycloakParams;
                    return { url, realm, clientId };
                })(),
                jwtClaimByUserKey
            };
            return () => out;
        })()),
        "getOrganizationUserProfileAttributeName": loggedProcedure.query((() => {
            const { organizationUserProfileAttributeName } = keycloakParams !== null && keycloakParams !== void 0 ? keycloakParams : {};
            if (organizationUserProfileAttributeName === undefined) {
                return () => {
                    throw new server_1.TRPCError({ "code": "METHOD_NOT_SUPPORTED" });
                };
            }
            return () => organizationUserProfileAttributeName;
        })()),
        "getSoftwares": loggedProcedure.query(() => core.states.readWriteSillData.getSoftwares()),
        "getInstances": loggedProcedure.query(() => core.states.readWriteSillData.getInstances()),
        "getExternalSoftwareOptions": loggedProcedure
            .input(zod_1.z.object({
            "queryString": zod_1.z.string(),
            "language": zLanguage
        }))
            .query(({ ctx: { user }, input }) => {
            if (user === undefined) {
                //To prevent abuse.
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { queryString, language } = input;
            return core.functions.suggestionAndAutoFill.getSoftwareExternalDataOptionsWithPresenceInSill({
                queryString,
                language
            });
        }),
        "getSoftwareFormAutoFillDataFromExternalSoftwareAndOtherSources": loggedProcedure
            .input(zod_1.z.object({
            "externalId": zod_1.z.string()
        }))
            .query(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                //To prevent abuse.
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { externalId } = input;
            return core.functions.suggestionAndAutoFill.getSoftwareFormAutoFillDataFromExternalAndOtherSources({
                externalId
            });
        }),
        "createSoftware": loggedProcedure
            .input(zod_1.z.object({
            "formData": zSoftwareFormData
        }))
            .mutation(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { formData } = input;
            try {
                await core.functions.readWriteSillData.createSoftware({
                    formData,
                    "agent": {
                        "email": user.email,
                        "organization": user.organization
                    }
                });
            }
            catch (e) {
                throw new server_1.TRPCError({ "code": "INTERNAL_SERVER_ERROR", "message": String(e) });
            }
        }),
        "updateSoftware": loggedProcedure
            .input(zod_1.z.object({
            "softwareSillId": zod_1.z.number(),
            "formData": zSoftwareFormData
        }))
            .mutation(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { softwareSillId, formData } = input;
            await core.functions.readWriteSillData.updateSoftware({
                softwareSillId,
                formData,
                "agent": {
                    "email": user.email,
                    "organization": user.organization
                }
            });
        }),
        "createUserOrReferent": loggedProcedure
            .input(zod_1.z.object({
            "formData": zDeclarationFormData,
            "softwareName": zod_1.z.string()
        }))
            .mutation(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { formData, softwareName } = input;
            await core.functions.readWriteSillData.createUserOrReferent({
                formData,
                softwareName,
                "agent": {
                    "email": user.email,
                    "organization": user.organization
                }
            });
        }),
        "removeUserOrReferent": loggedProcedure
            .input(zod_1.z.object({
            "softwareName": zod_1.z.string(),
            "declarationType": zod_1.z.enum(["user", "referent"])
        }))
            .mutation(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { softwareName, declarationType } = input;
            await core.functions.readWriteSillData.removeUserOrReferent({
                softwareName,
                "agentEmail": user.email,
                declarationType
            });
        }),
        "createInstance": loggedProcedure
            .input(zod_1.z.object({
            "formData": zInstanceFormData
        }))
            .mutation(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { formData } = input;
            const { instanceId } = await core.functions.readWriteSillData.createInstance({
                formData,
                "agent": {
                    "email": user.email,
                    "organization": user.organization
                }
            });
            return { instanceId };
        }),
        "updateInstance": loggedProcedure
            .input(zod_1.z.object({
            "instanceId": zod_1.z.number(),
            "formData": zInstanceFormData
        }))
            .mutation(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { instanceId, formData } = input;
            await core.functions.readWriteSillData.updateInstance({
                instanceId,
                formData,
                "agentEmail": user.email
            });
        }),
        "getAgents": loggedProcedure.query(async ({ ctx: { user } }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const agents = core.states.readWriteSillData.getAgents();
            return { agents };
        }),
        "updateIsAgentProfilePublic": loggedProcedure
            .input(zod_1.z.object({
            "isPublic": zod_1.z.boolean()
        }))
            .mutation(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { isPublic } = input;
            await core.functions.readWriteSillData.updateIsAgentProfilePublic({
                "agent": {
                    "email": user.email,
                    "organization": user.organization
                },
                isPublic
            });
        }),
        "updateAgentAbout": loggedProcedure
            .input(zod_1.z.object({
            "about": zod_1.z.string().optional()
        }))
            .mutation(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { about } = input;
            await core.functions.readWriteSillData.updateAgentAbout({
                "agent": {
                    "email": user.email,
                    "organization": user.organization
                },
                about
            });
        }),
        "getIsAgentProfilePublic": loggedProcedure
            .input(zod_1.z.object({
            "email": zod_1.z.string()
        }))
            .query(async ({ input }) => {
            const { email } = input;
            const isPublic = core.functions.readWriteSillData.getAgentIsPublic({
                email
            });
            return { isPublic };
        }),
        "getAgent": loggedProcedure
            .input(zod_1.z.object({
            "email": zod_1.z.string()
        }))
            .query(async ({ ctx: { user }, input }) => {
            const { email } = input;
            const isPublic = core.functions.readWriteSillData.getAgentIsPublic({
                email
            });
            if (!isPublic && user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const agent = await core.functions.readWriteSillData.getAgent({
                email
            });
            return { agent };
        }),
        "getAllowedEmailRegexp": loggedProcedure.query(coreContext.userApi.getAllowedEmailRegexp),
        "getAllOrganizations": loggedProcedure.query(coreContext.userApi.getAllOrganizations),
        "changeAgentOrganization": loggedProcedure
            .input(zod_1.z.object({
            "newOrganization": zod_1.z.string()
        }))
            .mutation(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            (0, assert_1.assert)(keycloakParams !== undefined);
            const { newOrganization } = input;
            await core.functions.readWriteSillData.changeAgentOrganization({
                "email": user.email,
                newOrganization,
                "userId": user.id
            });
        }),
        "updateEmail": loggedProcedure
            .input(zod_1.z.object({
            "newEmail": zod_1.z.string().email()
        }))
            .mutation(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { newEmail } = input;
            (0, assert_1.assert)(keycloakParams !== undefined);
            await core.functions.readWriteSillData.updateUserEmail({
                "userId": user.id,
                "email": user.email,
                newEmail
            });
        }),
        "getRegisteredUserCount": loggedProcedure.query(async () => coreContext.userApi.getUserCount()),
        "getTotalReferentCount": loggedProcedure.query(() => {
            const referentCount = core.states.readWriteSillData.getReferentCount();
            return { referentCount };
        }),
        "getTermsOfServiceUrl": loggedProcedure.query(() => termsOfServiceUrl),
        "getMarkdown": loggedProcedure
            .input(zod_1.z.object({
            "language": zLanguage,
            "name": zod_1.z.union([zod_1.z.literal("readme"), zod_1.z.literal("termsOfService")])
        }))
            .query((() => {
            const maxAge = (1000 * 3600) / 2;
            const memoizedFetch = (0, memoizee_1.default)(async (url) => (0, node_fetch_1.default)(url).then(res => res.text()), {
                "promise": true,
                maxAge,
                "preFetch": true
            });
            // prettier-ignore
            GetSoftwareExternalData_1.languages
                .map(lang => (0, reactless_1.createResolveLocalizedString)({ "currentLanguage": lang, "fallbackLanguage": "en" }))
                .map(({ resolveLocalizedString }) => [termsOfServiceUrl, readmeUrl].map(resolveLocalizedString))
                .flat()
                .forEach(async function callee(url) {
                memoizedFetch(url);
                await new Promise(resolve => setTimeout(resolve, maxAge - 10000));
                callee(url);
            });
            return async ({ input }) => {
                const { language, name } = input;
                const { resolveLocalizedString } = (0, reactless_1.createResolveLocalizedString)({
                    "currentLanguage": language,
                    "fallbackLanguage": "en"
                });
                return memoizedFetch(resolveLocalizedString((() => {
                    switch (name) {
                        case "readme":
                            return readmeUrl;
                        case "termsOfService":
                            return termsOfServiceUrl;
                    }
                })()));
            };
        })()),
        "unreferenceSoftware": loggedProcedure
            .input(zod_1.z.object({
            "softwareName": zod_1.z.string(),
            "reason": zod_1.z.string()
        }))
            .mutation(async ({ ctx: { user }, input }) => {
            if (user === undefined) {
                throw new server_1.TRPCError({ "code": "UNAUTHORIZED" });
            }
            const { softwareName, reason } = input;
            await core.functions.readWriteSillData.unreferenceSoftware({
                softwareName,
                reason
            });
        })
    });
    return { router };
}
exports.createRouter = createRouter;
const zSoftwareType = zod_1.z.union([
    zod_1.z.object({
        "type": zod_1.z.literal("desktop/mobile"),
        "os": zod_1.z.object({
            "windows": zod_1.z.boolean(),
            "linux": zod_1.z.boolean(),
            "mac": zod_1.z.boolean(),
            "android": zod_1.z.boolean(),
            "ios": zod_1.z.boolean()
        })
    }),
    zod_1.z.object({
        "type": zod_1.z.literal("cloud")
    }),
    zod_1.z.object({
        "type": zod_1.z.literal("stack")
    })
]);
{
    (0, assert_1.assert)();
}
const zOs = zod_1.z.enum(["windows", "linux", "mac", "android", "ios"]);
{
    (0, assert_1.assert)();
}
const zSoftwareFormData = (() => {
    const zOut = zod_1.z.object({
        "softwareType": zSoftwareType,
        "externalId": zod_1.z.string().optional(),
        "comptoirDuLibreId": zod_1.z.number().optional(),
        "softwareName": zod_1.z.string(),
        "softwareDescription": zod_1.z.string(),
        "softwareLicense": zod_1.z.string(),
        "softwareMinimalVersion": zod_1.z.string(),
        "isPresentInSupportContract": zod_1.z.boolean(),
        "isFromFrenchPublicService": zod_1.z.boolean(),
        "similarSoftwareExternalDataIds": zod_1.z.array(zod_1.z.string()),
        "softwareLogoUrl": zod_1.z.string().optional(),
        "softwareKeywords": zod_1.z.array(zod_1.z.string()),
        "doRespectRgaa": zod_1.z.boolean().or(zod_1.z.null())
    });
    {
        (0, assert_1.assert)();
    }
    return zOut;
})();
const zDeclarationFormData = (() => {
    const zUser = zod_1.z.object({
        "declarationType": zod_1.z.literal("user"),
        "usecaseDescription": zod_1.z.string(),
        "os": zOs.optional(),
        "version": zod_1.z.string(),
        "serviceUrl": zod_1.z.string().optional()
    });
    {
        (0, assert_1.assert)();
    }
    const zReferent = zod_1.z.object({
        "declarationType": zod_1.z.literal("referent"),
        "isTechnicalExpert": zod_1.z.boolean(),
        "usecaseDescription": zod_1.z.string(),
        "serviceUrl": zod_1.z.string().optional()
    });
    {
        (0, assert_1.assert)();
    }
    return zod_1.z.union([zUser, zReferent]);
})();
const zInstanceFormData = (() => {
    const zOut = zod_1.z.object({
        "mainSoftwareSillId": zod_1.z.number(),
        "organization": zod_1.z.string(),
        "targetAudience": zod_1.z.string(),
        "publicUrl": zod_1.z.string().optional(),
        "otherSoftwareWikidataIds": zod_1.z.array(zod_1.z.string())
    });
    {
        (0, assert_1.assert)();
    }
    return zOut;
})();
const zLanguage = zod_1.z.union([zod_1.z.literal("fr"), zod_1.z.literal("en")]);
{
    (0, assert_1.assert)();
}
//# sourceMappingURL=router.js.map