"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startRpcService = void 0;
const express_1 = __importDefault(require("express"));
const trpcExpress = __importStar(require("@trpc/server/adapters/express"));
const cors_1 = __importDefault(require("cors"));
const validateGithubWebhookSignature_1 = require("../tools/validateGithubWebhookSignature");
const compression_1 = __importDefault(require("compression"));
const core_1 = require("../core");
const assert_1 = require("tsafe/assert");
const context_1 = require("./context");
const router_1 = require("./router");
const path_1 = require("path");
async function startRpcService(params) {
    const { redirectUrl, dataRepoSshUrl, sshPrivateKeyForGitName, sshPrivateKeyForGit, keycloakParams, termsOfServiceUrl, readmeUrl, jwtClaimByUserKey, githubWebhookSecret, port, githubPersonalAccessTokenForApiRateLimit, isDevEnvironnement, externalSoftwareDataOrigin, ...rest } = params;
    (0, assert_1.assert)();
    console.log({ isDevEnvironnement });
    const { core, context: coreContext } = await (0, core_1.bootstrapCore)({
        "gitDbApiParams": {
            dataRepoSshUrl,
            "sshPrivateKeyName": sshPrivateKeyForGitName,
            "sshPrivateKey": sshPrivateKeyForGit
        },
        "keycloakUserApiParams": keycloakParams === undefined
            ? undefined
            : {
                "url": keycloakParams.url,
                "realm": keycloakParams.realm,
                "adminPassword": keycloakParams.adminPassword,
                "organizationUserProfileAttributeName": keycloakParams.organizationUserProfileAttributeName
            },
        githubPersonalAccessTokenForApiRateLimit,
        "doPerPerformPeriodicalCompilation": !isDevEnvironnement && redirectUrl === undefined,
        "doPerformCacheInitialization": redirectUrl === undefined,
        "externalSoftwareDataOrigin": externalSoftwareDataOrigin
    });
    console.log("Core API initialized");
    const { createContext } = (0, context_1.createContextFactory)({
        jwtClaimByUserKey,
        "keycloakParams": keycloakParams === undefined
            ? undefined
            : {
                "url": keycloakParams.url,
                "realm": keycloakParams.realm,
                "clientId": keycloakParams.clientId
            }
    });
    const { router } = (0, router_1.createRouter)({
        core,
        coreContext,
        jwtClaimByUserKey,
        "keycloakParams": keycloakParams === undefined
            ? undefined
            : {
                "url": keycloakParams.url,
                "realm": keycloakParams.realm,
                "clientId": keycloakParams.clientId,
                "organizationUserProfileAttributeName": keycloakParams.organizationUserProfileAttributeName
            },
        termsOfServiceUrl,
        readmeUrl,
        redirectUrl,
        externalSoftwareDataOrigin
    });
    (0, express_1.default)()
        .use((0, cors_1.default)())
        .use((0, compression_1.default)())
        .use((req, _res, next) => { var _a; return (console.log("⬅", req.method, req.path, (_a = req.body) !== null && _a !== void 0 ? _a : req.query), next()); })
        .use("/public/healthcheck", (...[, res]) => res.sendStatus(200))
        .post(`*/ondataupdated`, (() => {
        if (githubWebhookSecret === undefined) {
            return async (...[, res]) => res.sendStatus(410);
        }
        const { validateGitHubWebhookSignature } = (0, validateGithubWebhookSignature_1.createValidateGitHubWebhookSignature)({
            githubWebhookSecret
        });
        return async (req, res) => {
            const reqBody = await validateGitHubWebhookSignature(req, res);
            console.log("Webhook signature OK");
            if (redirectUrl !== undefined) {
                console.log("Doing nothing with the webhook, this instance is effectively disabled");
            }
            if (reqBody.ref !== `refs/heads/main`) {
                console.log(`Not a push on the main branch, doing nothing`);
                res.sendStatus(200);
                return;
            }
            console.log("Push on main branch of data repo");
            core.functions.readWriteSillData.notifyPushOnMainBranch({
                "commitMessage": reqBody.head_commit.message
            });
            res.sendStatus(200);
        };
    })())
        .get(`*/sill.json`, (req, res) => {
        if (redirectUrl !== undefined) {
            return res.redirect(redirectUrl + req.originalUrl);
        }
        const compiledDataPublicJson = core.states.readWriteSillData.getCompiledDataPublicJson();
        res.setHeader("Content-Type", "application/json").send(Buffer.from(compiledDataPublicJson, "utf8"));
    })
        .use((() => {
        const trpcMiddleware = trpcExpress.createExpressMiddleware({ router, createContext });
        return (req, res, next) => {
            const proxyReq = new Proxy(req, {
                get: (target, prop) => {
                    if (prop === "path") {
                        return `/${(0, path_1.basename)(target.path)}`;
                    }
                    return Reflect.get(target, prop);
                }
            });
            return trpcMiddleware(proxyReq, res, next);
        };
    })())
        .listen(port, () => console.log(`Listening on port ${port}`));
}
exports.startRpcService = startRpcService;
//# sourceMappingURL=start.js.map