"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NpmModuleVersion = void 0;
var NpmModuleVersion;
(function (NpmModuleVersion) {
    function parse(versionStr) {
        const match = versionStr.match(/^([0-9]+)\.([0-9]+)\.([0-9]+)(?:-beta.([0-9]+))?/);
        if (!match) {
            throw new Error(`${versionStr} is not a valid NPM version`);
        }
        return {
            "major": parseInt(match[1]),
            "minor": parseInt(match[2]),
            "patch": parseInt(match[3]),
            ...(() => {
                const str = match[4];
                return str === undefined ? {} : { "betaPreRelease": parseInt(str) };
            })()
        };
    }
    NpmModuleVersion.parse = parse;
    function stringify(v) {
        return `${v.major}.${v.minor}.${v.patch}${v.betaPreRelease === undefined ? "" : `-beta.${v.betaPreRelease}`}`;
    }
    NpmModuleVersion.stringify = stringify;
    /**
     *
     * v1  <  v2  => -1
     * v1 === v2  => 0
     * v1  >  v2  => 1
     *
     */
    function compare(v1, v2) {
        const sign = (diff) => (diff === 0 ? 0 : diff < 0 ? -1 : 1);
        const noUndefined = (n) => n !== null && n !== void 0 ? n : Infinity;
        for (const level of ["major", "minor", "patch", "betaPreRelease"]) {
            if (noUndefined(v1[level]) !== noUndefined(v2[level])) {
                return sign(noUndefined(v1[level]) - noUndefined(v2[level]));
            }
        }
        return 0;
    }
    NpmModuleVersion.compare = compare;
    /*
    console.log(compare(parse("3.0.0-beta.3"), parse("3.0.0")) === -1 )
    console.log(compare(parse("3.0.0-beta.3"), parse("3.0.0-beta.4")) === -1 )
    console.log(compare(parse("3.0.0-beta.3"), parse("4.0.0")) === -1 )
    */
    function bumpType(params) {
        const versionAhead = parse(params.versionAheadStr);
        const versionBehind = parse(params.versionBehindStr);
        if (compare(versionBehind, versionAhead) === 1) {
            throw new Error(`Version regression ${versionBehind} -> ${versionAhead}`);
        }
        for (const level of ["major", "minor", "patch", "betaPreRelease"]) {
            if (versionBehind[level] !== versionAhead[level]) {
                return level;
            }
        }
        return "same";
    }
    NpmModuleVersion.bumpType = bumpType;
})(NpmModuleVersion = exports.NpmModuleVersion || (exports.NpmModuleVersion = {}));
//# sourceMappingURL=NpmModuleVersion.js.map