"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createValidateKeycloakSignature = void 0;
const url_join_1 = __importDefault(require("url-join"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const memoizee_1 = __importDefault(require("memoizee"));
const keycloak_backend_1 = __importDefault(require("keycloak-backend"));
const assert_1 = require("tsafe/assert");
function createValidateKeycloakSignature(params) {
    const { url, realm, clientId } = params;
    const getKeycloakBackendVerifyOffline = (0, memoizee_1.default)(async () => {
        const cert = await fetchKeycloakRealmPublicCert({ url, realm });
        const keycloakBackend = (0, keycloak_backend_1.default)({
            realm,
            "auth-server-url": url.replace("/auth", ""),
            "client_id": clientId
        });
        async function keycloakBackendVerifyOffline(params) {
            const { keycloakOidcAccessToken } = params;
            const o = await keycloakBackend.jwt.verifyOffline(keycloakOidcAccessToken, cert);
            (0, assert_1.assert)(!o.isExpired(), "Token is expired");
        }
        return { keycloakBackendVerifyOffline };
    }, { "promise": true });
    async function validateKeycloakSignature(params) {
        const { accessToken } = params;
        const { keycloakBackendVerifyOffline } = await getKeycloakBackendVerifyOffline();
        await keycloakBackendVerifyOffline({
            "keycloakOidcAccessToken": accessToken
        });
    }
    return { validateKeycloakSignature };
}
exports.createValidateKeycloakSignature = createValidateKeycloakSignature;
async function fetchKeycloakRealmPublicCert(params) {
    const { url, realm } = params;
    const obj = await (0, node_fetch_1.default)((0, url_join_1.default)(url, "realms", realm, "protocol/openid-connect/certs")).then(res => res.json());
    return [
        "-----BEGIN CERTIFICATE-----",
        obj["keys"].find(({ use }) => use === "sig")["x5c"][0],
        "-----END CERTIFICATE-----"
    ].join("\n");
}
//# sourceMappingURL=createValidateKeycloakSignature.js.map