"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLatestSemVersionedTagFromSourceUrl = void 0;
const parseGithubRepoUrl_1 = require("./parseGithubRepoUrl");
const listTags_1 = require("./octokit-addons/listTags");
const rest_1 = require("@octokit/rest");
const semver = __importStar(require("semver"));
const parse_1 = __importDefault(require("semver/functions/parse"));
const exclude_1 = require("tsafe/exclude");
const id_1 = require("tsafe/id");
const assert_1 = require("tsafe/assert");
const graphql_1 = require("@octokit/graphql");
const node_fetch_1 = __importDefault(require("node-fetch"));
/** NOTE: This function can't throw */
async function getLatestSemVersionedTagFromSourceUrl(params) {
    const { sourceUrl, githubPersonalAccessTokenForApiRateLimit, isQuick } = params;
    let parsedGitHubRepoUrl;
    try {
        parsedGitHubRepoUrl = (0, parseGithubRepoUrl_1.parseGitHubRepoUrl)(sourceUrl);
    }
    catch {
        return undefined;
    }
    const octokit = new rest_1.Octokit({ "auth": githubPersonalAccessTokenForApiRateLimit, "request": { fetch: node_fetch_1.default } });
    if (githubPersonalAccessTokenForApiRateLimit === undefined) {
        return undefined;
    }
    const tags = isQuick
        ? []
        : await (async () => {
            const tags = [];
            const { listTags } = (0, listTags_1.listTagsFactory)({
                octokit
            });
            const asyncIterator = listTags({
                "owner": parsedGitHubRepoUrl.owner,
                "repo": parsedGitHubRepoUrl.repoName
            });
            const start = Date.now();
            async_iter: for await (const tag of asyncIterator) {
                if (Date.now() - start > 30 * 1000) {
                    console.log(`Listing ${sourceUrl} tags with octokit REST is taking too long, aborting...`);
                    return [];
                }
                for (const search of ["rc.", "alpha", "beta", "nightly", "canary", "pre", "wip", "next."]) {
                    if (tag.toLowerCase().includes(search)) {
                        continue async_iter;
                    }
                }
                if (/rc[0-9]/.test(tag)) {
                    continue;
                }
                tags.push(tag);
            }
            return tags;
        })().catch(() => (0, id_1.id)([]));
    const semverTags_valid = tags
        .map(tag => {
        const version = semver.valid(tag);
        if (version === null) {
            return undefined;
        }
        return {
            tag,
            version
        };
    })
        .filter((0, exclude_1.exclude)(undefined))
        .sort((a, b) => semver.compare(a.version, b.version));
    const semverTags_coerce = tags
        .filter(tag => !semverTags_valid.map(({ tag }) => tag).includes(tag))
        .map(tag => {
        const version = semver.valid(semver.coerce(tag));
        if (version === null) {
            return undefined;
        }
        {
            const parsedVersion = (0, parse_1.default)(version);
            (0, assert_1.assert)(parsedVersion !== null);
            if (parsedVersion.patch === 0 &&
                parsedVersion.minor === 0 &&
                !tag
                    .replace(/^[^0-9]*/, "")
                    .replace(/[^0-9.]/g, "")
                    .includes(".")) {
                return undefined;
            }
        }
        return {
            tag,
            version
        };
    })
        .filter((0, exclude_1.exclude)(undefined))
        .sort((a, b) => semver.compare(a.version, b.version));
    const [latestSemverTag_valid, latestSemverTag_coerce] = await Promise.all([semverTags_valid, semverTags_coerce].map(async (entries) => {
        if (entries.length === 0) {
            return undefined;
        }
        const { tag, version } = entries[entries.length - 1];
        const date = await getTagDate({
            "owner": parsedGitHubRepoUrl.owner,
            "repo": parsedGitHubRepoUrl.repoName,
            tag,
            octokit
        });
        if (date === null) {
            return undefined;
        }
        return {
            version,
            tag,
            date
        };
    }));
    const latestSemverTag = (() => {
        if (latestSemverTag_valid === undefined) {
            if (latestSemverTag_coerce === undefined) {
                return undefined;
            }
            return latestSemverTag_coerce;
        }
        if (latestSemverTag_coerce === undefined) {
            return latestSemverTag_valid;
        }
        if (latestSemverTag_coerce.date.getTime() - latestSemverTag_valid.date.getTime() > 2 * 24 * 3600 * 1000) {
            return latestSemverTag_coerce;
        }
        return latestSemverTag_valid;
    })();
    const latestReleaseInfo = await getLatestReleaseInfo({
        "owner": parsedGitHubRepoUrl.owner,
        "repo": parsedGitHubRepoUrl.repoName,
        octokit
    });
    if (latestReleaseInfo !== undefined &&
        (latestSemverTag === undefined ||
            latestReleaseInfo.publishedAt.getTime() - latestSemverTag.date.getTime() >
                /* three month */ 3 * 30 * 24 * 3600 * 1000)) {
        //console.log(`${sourceUrl} Returning release: ${JSON.stringify(latestReleaseInfo, null, 2)}`);
        return {
            "version": latestReleaseInfo.tag,
            "publicationTime": latestReleaseInfo.publishedAt.getTime()
        };
    }
    const allTagsWithDates = tags.length === 0
        ? null
        : await getAllTagsWithDatesUsingGraphQLApi({
            "owner": parsedGitHubRepoUrl.owner,
            "name": parsedGitHubRepoUrl.repoName,
            "githubToken": githubPersonalAccessTokenForApiRateLimit
        });
    if (allTagsWithDates !== null) {
        const latestTagWithDate = allTagsWithDates
            //sort by date, the most recent first
            .sort((a, b) => b.date.getTime() - a.date.getTime())[0];
        if (latestTagWithDate !== undefined &&
            (latestSemverTag === undefined ||
                latestTagWithDate.date.getTime() - latestSemverTag.date.getTime() >
                    /* three month */ 3 * 30 * 24 * 3600 * 1000)) {
            //console.log(`${sourceUrl} Returning GraphQL result: ${JSON.stringify(latestTagWithDate, null, 2)}`);
            return {
                "version": latestTagWithDate.name,
                "publicationTime": latestTagWithDate.date.getTime()
            };
        }
    }
    if (latestSemverTag === undefined) {
        return undefined;
    }
    //console.log(`${sourceUrl} Returning: ${JSON.stringify(latestSemverTag, null, 2)}`);
    return {
        "version": latestSemverTag.version,
        "publicationTime": latestSemverTag.date.getTime()
    };
}
exports.getLatestSemVersionedTagFromSourceUrl = getLatestSemVersionedTagFromSourceUrl;
async function getTagDate(params) {
    const { owner, repo, tag, octokit } = params;
    try {
        // Fetch the tag
        const tagRef = await octokit.git.getRef({
            owner,
            repo,
            "ref": `tags/${tag}`
        });
        // If tag is an annotated tag, it will have its own date, otherwise fetch the commit associated with the tag
        if (tagRef.data.object.type === "tag") {
            const tag = await octokit.git.getTag({
                owner,
                repo,
                tag_sha: tagRef.data.object.sha
            });
            return new Date(tag.data.tagger.date);
        }
        else if (tagRef.data.object.type === "commit") {
            const commit = await octokit.git.getCommit({
                owner,
                repo,
                commit_sha: tagRef.data.object.sha
            });
            return new Date(commit.data.author.date);
        }
    }
    catch (error) {
        console.error(`Failed to get the date for tag ${tag} in repo ${owner}/${repo}:`, error);
    }
    return null;
}
async function getLatestReleaseInfo(params) {
    var _a, _b;
    const { owner, repo, octokit } = params;
    try {
        const { data: releases } = await octokit.repos.listReleases({
            owner,
            repo,
            per_page: 1,
            page: 1
        });
        if (releases.length > 0) {
            const tag = (_a = releases[0]) === null || _a === void 0 ? void 0 : _a.tag_name;
            (0, assert_1.assert)(typeof tag === "string");
            const publishedAtStr = (_b = releases[0]) === null || _b === void 0 ? void 0 : _b.published_at;
            (0, assert_1.assert)(typeof publishedAtStr === "string");
            return {
                tag,
                publishedAt: new Date(publishedAtStr)
            };
        }
        else {
            return undefined;
        }
    }
    catch (error) {
        console.error(`Error fetching the latest release:${String(error)}`);
        return undefined;
    }
}
// NOTE: GPT generated
const getAllTagsWithDatesUsingGraphQLApi = async (params) => {
    const { owner, name, githubToken } = params;
    const query = `
    query($owner: String!, $name: String!, $after: String) {
      repository(owner: $owner, name: $name) {
        refs(refPrefix: "refs/tags/", first: 100, after: $after, orderBy: {field: TAG_COMMIT_DATE, direction: DESC}) {
          nodes {
            name
            target {
              ... on Tag {
                tagger {
                  date
                }
              }
            }
          }
          pageInfo {
            endCursor
            hasNextPage
          }
        }
      }
    }
  `;
    let tagsWithDates = [];
    let hasNextPage = true;
    let afterCursor;
    const start = Date.now();
    try {
        while (hasNextPage) {
            if (Date.now() - start > 30 * 1000) {
                console.log(`Listing ${owner}/${name} tags with octokit GraphQL is taking too long, aborting...`);
                return null;
            }
            const data = await (0, graphql_1.graphql)(query, {
                "headers": {
                    "authorization": `token ${githubToken}`
                },
                "request": {
                    fetch: node_fetch_1.default
                },
                owner,
                name,
                "after": afterCursor
            });
            const newTags = data.repository.refs.nodes
                .map((node) => {
                const date = node.target.tagger ? new Date(node.target.tagger.date) : undefined;
                if (date == undefined) {
                    return undefined;
                }
                return {
                    name: node.name,
                    date
                };
            })
                .filter((0, exclude_1.exclude)(undefined));
            tagsWithDates = [...tagsWithDates, ...newTags];
            hasNextPage = data.repository.refs.pageInfo.hasNextPage;
            afterCursor = data.repository.refs.pageInfo.endCursor;
        }
        if (tagsWithDates.length === 0) {
            return null;
        }
        return tagsWithDates;
    }
    catch (error) {
        console.error(`An error occurred while running the graphql query: ${error}`);
        return null;
    }
};
//# sourceMappingURL=getLatestSemVersionedTagFromSourceUrl.js.map