"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorNoBranch = exports.gitSsh = void 0;
const exec_1 = require("./exec");
const path_1 = require("path");
const fs = __importStar(require("fs"));
const crypto_1 = __importDefault(require("crypto"));
const async_mutex_1 = require("async-mutex");
const mutexes = {};
const globalMutex = new async_mutex_1.Mutex();
let isFirstCall = true;
const gitSsh = async (params) => {
    var _a;
    var _b;
    const { sshUrl, sshPrivateKeyName, sshPrivateKey, shaish, doForceReClone = false, commitAuthorEmail = "actions@github.com", action } = params;
    const cacheDir = (0, path_1.join)(process.cwd(), "node_modules", ".cache", "gitSSH");
    await globalMutex.runExclusive(async () => {
        if (!isFirstCall) {
            return;
        }
        isFirstCall = false;
        await fs.promises.rm(cacheDir, { "recursive": true, "force": true });
        await fs.promises.mkdir(cacheDir, { "recursive": true });
    });
    const mutex = ((_a = mutexes[_b = sshUrl + (shaish || "")]) !== null && _a !== void 0 ? _a : (mutexes[_b] = new async_mutex_1.Mutex()));
    return mutex.runExclusive(async function callee() {
        await configureOpenSshClient({ sshPrivateKeyName, sshPrivateKey });
        const repoHash = crypto_1.default
            .createHash("sha1")
            .update(sshUrl + (shaish || ""))
            .digest("hex");
        const repoPath = (0, path_1.join)(cacheDir, repoHash);
        const repoExists = await fs.promises
            .stat(repoPath)
            .then(() => true)
            .catch(() => false);
        if (doForceReClone && repoExists) {
            await fs.promises.rm(repoPath, { "recursive": true, "force": true });
            await callee();
            return;
        }
        if (!repoExists) {
            console.log(`Performing git clone ${sshUrl} ${repoPath}`);
            if (shaish === undefined) {
                await (0, exec_1.exec)(`git clone --depth 1 ${sshUrl} ${repoPath}`);
            }
            else {
                if (isSha(shaish)) {
                    await (0, exec_1.exec)(`git clone ${sshUrl} ${repoPath}`);
                    await (0, exec_1.exec)(`git checkout ${shaish}`, { "cwd": repoPath });
                }
                else {
                    await (0, exec_1.exec)(`git clone --branch ${shaish} --depth 1 ${sshUrl} ${repoPath}`);
                }
            }
        }
        else {
            console.log(`Performing git pull ${sshUrl} ${repoPath}`);
            try {
                await (0, exec_1.exec)(`git pull`, { "cwd": repoPath });
                // NOTE: If on main or on a branch
                if (shaish === undefined || !isSha(shaish)) {
                    // NOTE: Exit gratuitously if there are no changes
                    await (0, exec_1.exec)([
                        `current_branch=$(git rev-parse --abbrev-ref HEAD)`,
                        `[ -z "$(git status --porcelain)" ]`,
                        `[ -z "$(git status --porcelain)" ]`,
                        `[ "$(git rev-list HEAD...origin/$current_branch --count)" -eq 0 ]`
                    ].join(" && "), { "cwd": repoPath });
                }
            }
            catch {
                console.log(`There's been a force push, so we're going to re-clone the repo ${sshUrl} ${repoPath}`);
                await fs.promises.rm(repoPath, { "recursive": true, "force": true });
                await callee();
                return;
            }
        }
        const changesResult = await action({ repoPath });
        if (changesResult.doCommit) {
            await (0, exec_1.exec)(`git config --local user.email "${commitAuthorEmail}"`, {
                "cwd": repoPath
            });
            await (0, exec_1.exec)(`git config --local user.name "${commitAuthorEmail.split("@")[0]}"`, { "cwd": repoPath });
            if (changesResult.doAddAll) {
                await (0, exec_1.exec)(`git add -A`, { "cwd": repoPath });
            }
            //NOTE: This can fail if there are no changes to commit
            try {
                await (0, exec_1.exec)(`git commit -am "${changesResult.message}"`, { "cwd": repoPath });
                await (0, exec_1.exec)(`git push`, { "cwd": repoPath });
            }
            catch { }
        }
    });
};
exports.gitSsh = gitSsh;
class ErrorNoBranch extends Error {
    constructor(message) {
        super(message);
        Object.setPrototypeOf(this, new.target.prototype);
    }
}
exports.ErrorNoBranch = ErrorNoBranch;
async function configureOpenSshClient(params) {
    const { sshPrivateKey, sshPrivateKeyName } = params;
    const sshConfigDirPath = (await (0, exec_1.exec)(`cd ~ && mkdir -p .ssh && cd .ssh && pwd`)).replace(/\r?\n$/, "");
    await fs.promises.writeFile((0, path_1.join)(sshConfigDirPath, sshPrivateKeyName), Buffer.from(sshPrivateKey.replace(/\\n/g, "\n"), "utf8"), { "mode": 0o600 });
    const sshConfigFilePath = (0, path_1.join)(sshConfigDirPath, "config");
    const doesSshConfigFileExists = !!(await fs.promises.stat(sshConfigFilePath).catch(() => null));
    if (doesSshConfigFileExists) {
        return;
    }
    await fs.promises.writeFile(sshConfigFilePath, Buffer.from("StrictHostKeyChecking=no\n", "utf8"));
}
function isSha(shaish) {
    return /^[0-9a-f]{7,40}$/i.test(shaish);
}
//# sourceMappingURL=gitSsh.js.map