"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createKeycloakAdminApiClient = void 0;
const url_join_1 = __importDefault(require("url-join"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const deepmerge_1 = __importDefault(require("deepmerge"));
function createKeycloakAdminApiClient(params) {
    const { url, adminPassword, realm } = params;
    return {
        "updateUser": async (params) => {
            const { userId, body } = params;
            const token = await obtainKeycloakAdminAccessToken({
                url,
                adminPassword
            });
            const endpointUrl = (0, url_join_1.default)(url, `admin/realms/${realm}/users/${userId}`);
            const currentBody = await (0, node_fetch_1.default)(endpointUrl, {
                "method": "GET",
                "headers": {
                    "Authorization": `Bearer ${token}`
                }
            }).then(async (resp) => {
                if (`${resp.status}`[0] !== "2") {
                    throw new Error(await resp.text());
                }
                return resp.json();
            });
            await (0, node_fetch_1.default)(endpointUrl, {
                "method": "PUT",
                "headers": {
                    "Content-Type": "application/json",
                    "Authorization": `Bearer ${token}`
                },
                "body": JSON.stringify((0, deepmerge_1.default)(currentBody, body))
            }).then(async (resp) => {
                const out = await resp.text();
                if (`${resp.status}`[0] !== "2") {
                    throw new Error(out);
                }
                return out;
            });
        },
        "getUserProfileAttributes": async () => {
            const { attributes } = await (0, node_fetch_1.default)((0, url_join_1.default)(url, `admin/realms/${realm}/users/profile`), {
                "method": "GET",
                "headers": {
                    "Authorization": `Bearer ${await obtainKeycloakAdminAccessToken({ url, adminPassword })}`
                }
            }).then(async (resp) => {
                if (`${resp.status}`[0] !== "2") {
                    throw new Error(await resp.text());
                }
                return resp.json();
            });
            return attributes;
        },
        "getUsers": async ({ first, max }) => (0, node_fetch_1.default)((0, url_join_1.default)(url, `admin/realms/${realm}/users?${Object.entries({
            first,
            max
        })
            .map(([key, value]) => `${key}=${value}`)
            .join("&")}`), {
            "method": "GET",
            "headers": {
                "Authorization": `Bearer ${await obtainKeycloakAdminAccessToken({ url, adminPassword })}`
            }
        }).then(async (resp) => {
            if (`${resp.status}`[0] !== "2") {
                throw new Error(await resp.text());
            }
            return resp.json();
        })
    };
}
exports.createKeycloakAdminApiClient = createKeycloakAdminApiClient;
async function obtainKeycloakAdminAccessToken(params) {
    const { url, adminPassword } = params;
    const { access_token } = await (0, node_fetch_1.default)((0, url_join_1.default)(url, "realms/master/protocol/openid-connect/token"), {
        "method": "POST",
        "headers": {
            "Content-Type": "application/x-www-form-urlencoded"
        },
        "body": Object.entries({
            "client_id": "admin-cli",
            "username": "admin",
            "password": adminPassword,
            "grant_type": "password"
        })
            .map(([key, value]) => `${key}=${value}`)
            .join("&")
    }).then(async (resp) => {
        if (`${resp.status}`[0] !== "2") {
            throw new Error(await resp.text());
        }
        return resp.json();
    });
    return access_token;
}
//# sourceMappingURL=keycloakAdminApiClient.js.map