"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCommitAsyncIterableFactory = void 0;
const per_page = 30;
/** Iterate over the commits of a repo's branch */
function getCommitAsyncIterableFactory(params) {
    const { octokit } = params;
    function getCommitAsyncIterable(params) {
        const { owner, repo, branch } = params;
        let commits = [];
        let page = 0;
        let isLastPage = undefined;
        const getReposListCommitsResponseData = (params) => octokit.repos
            .listCommits({
            owner,
            repo,
            per_page,
            "page": params.page,
            "sha": branch
        })
            .then(({ data }) => data);
        return {
            [Symbol.asyncIterator]() {
                return {
                    "next": async () => {
                        if (commits.length === 0) {
                            if (isLastPage) {
                                return { "done": true, "value": undefined };
                            }
                            page++;
                            commits = await getReposListCommitsResponseData({
                                page
                            });
                            if (commits.length === 0) {
                                return { "done": true, "value": undefined };
                            }
                            isLastPage =
                                commits.length !== per_page ||
                                    (await getReposListCommitsResponseData({
                                        "page": page + 1
                                    })).length === 0;
                        }
                        const [commit, ...rest] = commits;
                        commits = rest;
                        return {
                            "value": commit,
                            "done": false
                        };
                    }
                };
            }
        };
    }
    return { getCommitAsyncIterable };
}
exports.getCommitAsyncIterableFactory = getCommitAsyncIterableFactory;
//# sourceMappingURL=getCommitAsyncIterable.js.map