"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCommonOriginFactory = void 0;
const getCommitAsyncIterable_1 = require("./getCommitAsyncIterable");
/** Return the sha of the first common commit between two branches */
function getCommonOriginFactory(params) {
    const { octokit } = params;
    const { getCommitAsyncIterable } = (0, getCommitAsyncIterable_1.getCommitAsyncIterableFactory)({
        octokit
    });
    async function getCommonOrigin(params) {
        const { owner, repo, branch1, branch2 } = params;
        const [commitAsyncIterable1, commitAsyncIterable2] = [branch1, branch2].map(branch => getCommitAsyncIterable({ owner, repo, branch }));
        const shas1 = [];
        const shas2 = [];
        for (;;) {
            const [itRes1, itRes2] = await Promise.all([commitAsyncIterable1, commitAsyncIterable2].map(commitAsyncIterable => commitAsyncIterable[Symbol.asyncIterator]().next()));
            let sha1 = undefined;
            if (!itRes1.done) {
                sha1 = itRes1.value.sha;
                shas1.push(sha1);
            }
            let sha2 = undefined;
            if (!itRes2.done) {
                sha2 = itRes2.value.sha;
                shas2.push(sha2);
            }
            if (!!sha1 && shas2.includes(sha1)) {
                return { "sha": sha1 };
            }
            if (!!sha2 && shas1.includes(sha2)) {
                return { "sha": sha2 };
            }
            if (itRes1.done && itRes2.done) {
                throw new Error("No common origin");
            }
        }
    }
    return { getCommonOrigin };
}
exports.getCommonOriginFactory = getCommonOriginFactory;
//# sourceMappingURL=getCommonOrigin.js.map