"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLatestSemVersionedTagFactory = void 0;
const listTags_1 = require("./listTags");
const NpmModuleVersion_1 = require("../NpmModuleVersion");
function getLatestSemVersionedTagFactory(params) {
    const { octokit } = params;
    async function getLatestSemVersionedTag(params) {
        const { owner, repo, doIgnoreBeta } = params;
        const semVersionedTags = [];
        const { listTags } = (0, listTags_1.listTagsFactory)({ octokit });
        let tagsAlreadyListed = 0;
        for await (const tag of listTags({ owner, repo })) {
            if (tagsAlreadyListed === 15) {
                break;
            }
            tagsAlreadyListed++;
            let version;
            if (!(tag.includes(".") && /\d/.test(tag))) {
                continue;
            }
            try {
                version = NpmModuleVersion_1.NpmModuleVersion.parse(tag.replace(/^[vV]?/, ""));
            }
            catch {
                version = undefined;
            }
            if (doIgnoreBeta && version !== undefined && version.betaPreRelease !== undefined) {
                continue;
            }
            semVersionedTags.push({ tag, version });
        }
        return semVersionedTags[0];
    }
    return { getLatestSemVersionedTag };
}
exports.getLatestSemVersionedTagFactory = getLatestSemVersionedTagFactory;
//# sourceMappingURL=getLatestSemVersionedTag.js.map