"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPullRequestAsyncIterableFactory = void 0;
const per_page = 99;
function getPullRequestAsyncIterableFactory(params) {
    const { octokit } = params;
    function getPullRequestAsyncIterable(params) {
        const { owner, repo, state } = params;
        let pullRequests = [];
        let page = 0;
        let isLastPage = undefined;
        const getPullsListResponseData = (params) => octokit.pulls
            .list({
            owner,
            repo,
            state,
            per_page,
            "page": params.page
        })
            .then(({ data }) => data);
        return {
            [Symbol.asyncIterator]() {
                return {
                    "next": async () => {
                        if (pullRequests.length === 0) {
                            if (isLastPage) {
                                return { "done": true, "value": undefined };
                            }
                            page++;
                            pullRequests = await getPullsListResponseData({
                                page
                            });
                            if (pullRequests.length === 0) {
                                return { "done": true, "value": undefined };
                            }
                            isLastPage =
                                pullRequests.length !== per_page ||
                                    (await getPullsListResponseData({
                                        "page": page + 1
                                    })).length === 0;
                        }
                        const [pullRequest, ...rest] = pullRequests;
                        pullRequests = rest;
                        return {
                            "value": pullRequest,
                            "done": false
                        };
                    }
                };
            }
        };
    }
    return { getPullRequestAsyncIterable };
}
exports.getPullRequestAsyncIterableFactory = getPullRequestAsyncIterableFactory;
//# sourceMappingURL=getPullRequestAsyncIterable.js.map