"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.listCommitFactory = void 0;
const getCommitAsyncIterable_1 = require("./getCommitAsyncIterable");
/** Return the list of commit since given sha (excluded)
 * ordered from the oldest to the newest */
function listCommitFactory(params) {
    const { octokit } = params;
    const { getCommitAsyncIterable } = (0, getCommitAsyncIterable_1.getCommitAsyncIterableFactory)({
        octokit
    });
    async function listCommit(params) {
        const { owner, repo, branch, sha } = params;
        const commitAsyncIterable = getCommitAsyncIterable({
            owner,
            repo,
            branch
        });
        const commits = [];
        for await (const commit of commitAsyncIterable) {
            if (commit.sha === sha) {
                break;
            }
            commits.push(commit);
        }
        return commits.reverse();
    }
    return { listCommit };
}
exports.listCommitFactory = listCommitFactory;
//# sourceMappingURL=listCommit.js.map